﻿Param([string]$LogFile)

if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

#########################################################################################
##
function Configure-AGSMachine([string]$AGSSetup,[string]$AGSLicense,[string]$AzurePS,
                                                                    [string]$SQLClient,
                                                                    [string]$SSLCert,
                                                                    [string]$Firewall, 
                                                                    [switch]$CreateSite,[string]$AdminName,[string]$AdminPassword,
                                                                    [switch]$Sysprep,
                                                                    [switch]$SysprepAsynch,
                                                                    [string]$SignalOK="555-555")
{
  log "----- AGS Custom extension invoked -----"
  try 
  { 
    if ($AzurePS) {
      lnl 2
      log "Operation : {0}" -p "Installing Azure PS"  
      $file = Join-Path $pwd -ChildPath $AzurePS
      Install-AzurePS $file
      log "Removing Azure PS file : {0}" -p $file 
      Remove-Item -Path $file -Force -Verbose -ErrorAction SilentlyContinue
    }

    if ($AGSSetup) {
      lnl 2
      log "Operation : {0}" -p "Installing AGS"  
      $file = Join-Path $pwd -ChildPath $AGSSetup
      Install-ArcGISServer -AgsSetupPath $file

      $f2clean = @($AGSSetup,"7z.exe","7z.dll")
      foreach ($item in $f2clean) {
        $file = Join-Path $pwd -ChildPath $item
        if (Test-Path $file) {
          log "Removing AGS setup file : {0}" -p $file
          Remove-Item -Path $file -Force -Verbose -ErrorAction SilentlyContinue
        }
      }
    }

    if ($AGSLicense) {
      lnl 2
      log "Operation : {0}" -p "Authorizing AGS"  
      $file = Join-Path $pwd -ChildPath $AGSLicense
      Install-AGSLicense -LicFile $file
      log "Removing AGS license file : {0}" -p $file 
      Remove-Item -Path $file -Force -Verbose -ErrorAction SilentlyContinue
    }

    if ($SSLCert) {
      lnl 2
      log "Operation : {0}" -p "Installing SSL cerrtificate"  
      $file = Join-Path $pwd -ChildPath $SSLCert
      #    install SSL cert
      log " - NOT IMPLEMENTED - "
      log "Removing SSL cert file : {0}" -p $file 
      Remove-Item -Path $file -Force -Verbose -ErrorAction SilentlyContinue
    }

    if ($SQLClient) {
      lnl 2
      log "Operation : {0}" -p "Installing SQL Client Library"  
      $file = Join-Path $pwd -ChildPath $SQLClient
      Install-SQLClientLib $file
      log "Removing SQL Client Library file : {0}" -p $file 
      Remove-Item -Path $file -Force -Verbose -ErrorAction SilentlyContinue
    }

    if ($CreateSite) {
      lnl 2
      log "Operation : {0}" -p "Creating AGS site"  
      if (!$AdminName -or !$AdminPassword) {
        throw (fls "admin name and passwords are required to create a new site")
      }
      Start-AGSService
      log "admin name : {0}, admin password : {1}" -p $AdminName,"*****"
      Create-AGSSite -UserName $AdminName -Password $AdminPassword    # localhost
    }

    if ($Firewall) {
      lnl 2
      log "Operation : {0}" -p "Setting Firewall rules"  
      Setup-VMFirewall $Firewall # rules are hardcoded, no IP restricitons
    }

    if ($Sysprep) {
      lnl 2
      log "Operation : {0}" -p "Running sysprep synch"  
      Reset-SequenceNumberForCustomScriptExtension
      Run-Sysprep 
    } elseif ($SysprepAsynch) {
      lnl 2
      log "Operation : {0}" -p "Running sysprep asynch"  
      Reset-SequenceNumberForCustomScriptExtension
      Run-Sysprep -RunAsynch
    }

    log "AGS extension completed : {0}" -p $SignalOK
  }
  catch
  {
    log "Configure-AGSMachine.ps1 : FAILED" -fc Red
    $_ | logr
  }
  finally  
  {
    lnl 2
    log "CLEANING : {0}" -p $curDir
    
    $f2clean = @("Configure-AGSMachine.ps1","agsLogTools.ps1","agsAdminRESTTools.ps1")
    foreach ($item in $f2clean) {
      $file = Join-Path $curDir -ChildPath $item
      if (Test-Path $file) {
        log "Removing file : {0}" -p $file
        Remove-Item -Path $file -Force -Verbose -ErrorAction SilentlyContinue
      }
    }
  }
}

#########################################################################################
##
function Run-Sysprep([switch]$RunAsynch,[int]$Delay=20)
{
  $sysDir = [System.Environment]::ExpandEnvironmentVariables("%SystemDrive%\SysprepUnlimited")  
  if (!(Test-Path $sysDir)) { 
    Install-SysprepUnlimited $sysDir
  }
  if ($RunAsynch) 
  {
    log "invoking sysprep asynchronously, as a Task scheduled to start in {0} seconds" -p $Delay
    $taskName  = "AGSSysprep"
    $runTask   =  Join-Path $sysDir -ChildPath "RunSysprep.bat"
    log "Task name : {0}, exec : {1}" -p $taskName,$runTask

    $whenStart = (get-date) + (New-TimeSpan -Seconds $Delay)
    $startDate = '{0:MM/dd/yyyy}' -f $whenStart
    $startTime = '{0:HH:mm:ss}'   -f $whenStart

    $res = schtasks /create /sc once /tn $taskName /tr $runTask /st $startTime /sd $startDate /F /RU SYSTEM /RL HIGHEST
    log "start date : {0}, start time {1}" -p $startDate,$startTime
    log $res

    ###
    ### to remove the task later:
    ###     $res = SCHTASKS /Delete /TN $taskName /F
    ###     Log-Message $res
    ###
  } 
  else  ## Run synch
  {
    Set-Location  $sysDir
    $exec = Join-Path $sysDir -ChildPath "RunSysprep.bat"

    ## TRICK! Report OK now since we may never return from this point
    log "Invoking sysprep, machine will be stopped in 10-20 minutes : {0}" -p $SignalOK

    sleep -Seconds $Delay
    Start-Process  $exec -Wait
  }
}

#########################################################################################
##
function Install-SysprepUnlimited([string]$Path)
{
  Log-Message "Installing SysprepUnlimited into folder : {0}" -p $Path
  New-Item -Path $Path -Type directory -Force -ErrorAction SilentlyContinue | Out-Null
 
  ## TODO: check if this is really needed 
  $rule = New-Object System.Security.AccessControl.FileSystemAccessRule("Everyone","FullControl","ContainerInherit,ObjectInherit","None","Allow")
  $acl = Get-Acl $Path
  $acl.SetAccessRule($rule)
  Set-Acl $Path $acl

  ##
  ##  ResetSkipRearm.reg
  ##
  $fileName = "ResetSkipRearm.reg"
  Log-Message "creating {0}\{1}" -p $Path,$fileName
@'   
Windows Registry Editor Version 5.00

[HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\SoftwareProtectionPlatform]
"SkipRearm"=dword:00000001
'@ | Set-Content (Join-Path $Path -ChildPath $fileName) -Force

  ##
  ##  RunSysprep.bat
  ##
  $fileName = "RunSysprep.bat"
  Log-Message "creating {0}\{1}" -p $Path,$fileName
@'   
cd c:\SysprepUnlimited
regedit /S ResetSkipRearm.reg
cd %windir%\system32\sysprep
sysprep.exe /oobe /generalize /shutdown /quiet
'@ | Set-Content (Join-Path $Path -ChildPath $fileName) -Force

  Log-Message "SysprepUnlimited is installed"
}

#########################################################################################
##
function Install-ArcGISServer([string]$AgsSetupPath)
{
  if (!(Test-Path $AgsSetupPath)) {
    Log-Message "File not found : {0}" -p $AgsSetupPath
    # throw ?
    return $false
  } 

  if ($env:AGSSERVER) {
    log "ArcGIS Server is already installed : {0}" -p $env:AGSSERVER
    return $false
  }

  $tmpDir = Join-Path $pwd -ChildPath "tmp-ags-setup" # + session id ?

  ## setup path can point to ZIP, EXE or a folder
  if (Test-Path $AgsSetupPath -PathType Leaf) 
  {
    if ($AgsSetupPath.EndsWith(".zip")) 
    {
      Log-Message "uncompressing file {0} to folder {1}" -p $AgsSetupPath,$tmpDir
      New-Item $tmpDir -Type directory -Force -ErrorAction SilentlyContinue | Out-Null 
      [Reflection.Assembly]::LoadWithPartialName("System.IO.Compression.FileSystem")
      [System.IO.Compression.ZipFile]::ExtractToDirectory($AgsSetupPath,$tmpDir) | Out-Null
      $AgsSetupPath = $TmpDir
    } 
    elseif ($AgsSetupPath.EndsWith(".exe")) 
    {
      Log-Message "uncompressing file {0} to folder {1}" -p $AgsSetupPath,$tmpDir
      New-Item $tmpDir -Type directory -Force -ErrorAction SilentlyContinue | Out-Null 

      log "making sure 7z is available"
      $SevenZip = Join-Path $pwd -ChildPath "7z.exe"
      if (!(Test-Path $SevenZip)) {
        throw (fls "File not found : {0}" -p $SevenZip)
      }
      
      log "use 7z to un-pack original ags installer(exe) "
      Start-Process -FilePath $SevenZip -ArgumentList " x -y $AgsSetupPath -o$tmpDir" -Wait
      $AgsSetupPath = $TmpDir
    } 
    else { 
      throw (fls "ags setup path points to a file which does not appear to be a correct AGS installer : {0}" -p $AgsSetupPath)
    } 
  } 

  ## if we get here, $AgsSetupPath is pointing to a folder which may or may not contain setup files
  ##
  $exec = Get-ChildItem $AgsSetupPath -Recurse -Include Setup.exe | select -First 1
  if (!$exec) {
    throw (fls "setup.exe not found in any subfolder of {0}" -p $AgsSetupPath)
  }

  $user   = 'arcgis'
  $pswd   = "{" + ([System.Guid]::NewGuid()).Guid + "}"

  $log    = Join-Path (Get-LogDir) -ChildPath "install-ags.log"   
  $pPath  = [System.Environment]::ExpandEnvironmentVariables("%SystemDrive%\Python27")  
  $params = "/qn USER_NAME=$user PASSWORD={0} INSTALLDIR1=$pPath /log $log" 

  Log-Message "running setup"
  Log-Message "   exec     : {0}" -p $exec
  Log-Message "   params   : {0}" -p ($params -f "******")

  Start-Process $exec $($params -f $pswd) -Wait
  Log-Message "AGS Setup completed"

  # clean the tmp dir if needed
  if (Test-Path $tmpDir) {
    Remove-Item $tmpDir -Recurse -Force -ErrorAction SilentlyContinue | Out-Null
  } 

  return $true
}

#########################################################################################
##
function Install-AGSLicense([string]$LicFile,[string]$Version="10.2")
{
  Log-Message "ArcGIS Server authorization, license file : {0}" -p $LicFile
  if (!(Test-Path $LicFile)) {
    throw (fls "File not found : {0}" -p $LicFile)
  }

  $exec    = [System.Environment]::ExpandEnvironmentVariables("%CommonProgramFiles%\ArcGIS\bin\SoftwareAuthorization.exe")
  if (!(Test-Path $exec)) {
    throw (fls "File not found : {0}" -p $exec)
  }

  $params  = "/S /Ver $Version /LIF $LicFile"

  Log-Message "running exec   : {0}" -p $exec
  Log-Message "        params : {0}" -p $params

  Start-Process $exec $params -Wait 

  $keyFile = [System.Environment]::ExpandEnvironmentVariables("%ProgramFiles%\ESRI\License$Version\sysgen\keycodes")  
  if (Test-Path $keyFile) {
    Log-Message "SUCCEEDED: AGS license is succesfully installed"
    return $true
  } 
  
  Log-Message "FAILED: key file is not generated: $keyFile"
  return $false
}

##############################################################################
##
function Install-SQLClientLib([string]$MSIFile)
{
  ## TODO: check if already installed
  $envVarName  = "ALLOWGDBONAZURE"
  $envVarValue = "true"
  [Environment]::SetEnvironmentVariable("ALLOWGDBONAZURE", "true", "Machine")
  log "Environment variable is set : {0}={1}" -p $envVarName,$envVarValue

  $log    = Join-Path (Get-LogDir) -ChildPath "install-sqlcli.log"
  $params = "/qn IACCEPTSQLNCLILICENSETERMS=YES /Log $log"
  Start-Process $MSIFile $params -Wait
  ## TODO: confirm it was really installed
}

##############################################################################
##
function Setup-VMFirewall([string]$Firewall)
{
  Log-Message "Setup firewall for machine {0}" -p $env:COMPUTERNAME
  
  if ($Firewall -ieq "ON") {
    Log-Message "Firewall is ON. Opening ports 6080 and 6443 for incoming traffic"
    Create-FirewallRule -RuleName "ArcGISServer HTTP-IN"  -RuleString "6080:*" -Descr "ags input port, HTTP" 
    Create-FirewallRule -RuleName "ArcGISServer HTTPS-IN" -RuleString "6443:*" -Descr "ags input port, HTTPS" 
    Log-Message "Ports are now open."
    return $true  
  } 
  
  if ($Firewall -ieq "OFF") {
    Log-Message "Firewall is OFF. Closing ports 6080 and 6443 for incoming traffic"
    Remove-FirewallRule -RuleName "ArcGISServer HTTP-IN"
    Remove-FirewallRule -RuleName "ArcGISServer HTTPS-IN"
    Log-Message "Ports are now closed."
    return $true
  } 

  Log-Message "Unkown input : {0}" -p $Firewall
  return $false
}

#################################################################################
## remove all the existing rules 
##
function Remove-FirewallRule([string]$RuleName,$FWPolicyRef=$null)
{
  $fwPolicy2 = $FWPolicyRef
  if ($fwPolicy2 -eq $null) {
    $fwPolicy2 = New-object -ComObject HNetCfg.FwPolicy2 
  }

  $esRules   = $fwPolicy2.Rules | select-string -inputobject {$_.Name} -pattern $RuleName -SimpleMatch
  foreach ($item in $esRules) {
    if ($item -ne $null ) { 
      $fwPolicy2.Rules.Remove($item.ToString())
    }
  }
}

#################################################################################
## get all the new rules from host-config file
##
## <Ports>      is   80,6080,5000-5010
## <IPScope>    is   11.11.11.11,192.168.0.12,1.2.3.4-2.2.2.2,3.3.3.3-5.5.5.5
## $ruleString  is   <Ports>:<IPScope>
##
function Create-FirewallRule([string]$RuleName,[string]$RuleString,[string]$Descr="")
{
  $esPorts   = $RuleString.split(":")[0]  # <Ports>
  $esIPScope = $RuleString.split(":")[1]  # <IPScope>

  $esRule    = New-object -ComObject HNetCfg.Fwrule

  $esRule.Name            = $RuleName
  $esRule.Description     = $Descr
  $esRule.Protocol        = 6  # TCP
  $esRule.LocalPorts      = $esPorts
  $esRule.RemotePorts     = '*'
  $esRule.LocalAddresses  = '*'
  $esRule.RemoteAddresses = $esIPScope
  $esRule.Enabled         = $true

  $fwPolicy2 = New-object -ComObject HNetCfg.FwPolicy2 
#  Remove-FirewallRule -RuleName $RuleName -FWPolicyRef $fwPolicy2
  $fwPolicy2.Rules.Add($esRule)
}

##############################################################################
##
function Install-AzurePS([string]$MSIFile)
{
  if (!(Test-Path $MSIFile)) {
    log "File not found : {0}" -p $MSIFile
    return
  }

  ## check if already installed
  $azurePSDir = [System.Environment]::ExpandEnvironmentVariables("%ProgramFiles(x86)%\Microsoft SDKs\Windows Azure\PowerShell\ServiceManagement\Azure")
  if (Test-Path $azurePSDir) {
    log "Azure PS Module is already installed" 
    return 
  }

  $log = Join-Path (Get-LogDir) -ChildPath "install-AzurePS.log"
  $params = "/qn /Log $log"
  log "installing Azure PS Module : {0} {1}" -p $MSIFile,$params
  Start-Process $MSIFile $params -Wait
   
  log "importing Azure PS Module"
  Import-Module (Join-Path $azurePSDir -ChildPath "Azure.psd1") 
}

#########################################################################################
##
function Start-AGSService()
{
  $agsStatus = (Get-Service "ArcGIS Server").Status
  if ( $agsStatus -ne [System.ServiceProcess.ServiceControllerStatus]::Running ) 
  {
    Log-Message "Starting AGS service..."
    Start-Service "ArcGIS Server" 
    # it starts very fast, sleeping 1-2 seconds should be enough
    Sleep -Seconds 2
  
#    # waiting till it's actually started 
#    $svc = Get-Service "ArcGIS Server"
#    $svc.WaitForStatus([System.ServiceProcess.ServiceControllerStatus]::Running)  # waiting forever! 
#
# try {
#   $svc.WaitForStatus([System.ServiceProcess.ServiceControllerStatus]::Stopped,"00:00:05")   
# } catch { ------ interval expired ------- }
#
    Log-Message "AGS service is started"
  } 
}

#########################################################################################
## Workaround to reset the sequence number that is persisted in the Registty, to allow 
## custom script extension to work when a new VM is spun up from an image
function Reset-SequenceNumberForCustomScriptExtension()
{
    Remove-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows Azure\ScriptHandler' -Name 'MreSeqNumStarted' -ErrorAction Ignore
    Remove-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows Azure\ScriptHandler' -Name 'MreSeqNumFinished' -ErrorAction Ignore

    ### Unsure if this is needed. TODO:- Remove after confirmation
    [bool]$Done = $false
    [string]$ExtensionDir = $null
    [string]$CurrentPath = $(Get-Location).Path
    while(-not $Done){
        if('Downloads' -ieq (Split-Path $CurrentPath -Leaf)){
            $ExtensionDir = Split-Path $CurrentPath -Parent
            $Done = $true
        }
        else {
            $CurrentPath = Split-Path $CurrentPath -Parent
        }
    }   
    if($ExtensionDir -ne $null) {
        Get-ChildItem (Join-Path $ExtensionDir 'RuntimeSettings') |  ForEach-Object {
           Remove-Item $_.FullName -Force -ErrorAction Ignore | Out-Null
        }
    }     
}

#########################################################################################
##
##  main entry point
##
#########################################################################################
$curDir = Split-Path ((Get-Variable MyInvocation -Scope 0).Value.MyCommand.Path)

if ($Args -and ($Args.Count -gt 0))  # to allow dot-source inclusion
{
. "$curDir\agsAdminRESTTools.ps1"

  Log-Init $LogFile

  Set-Location $curDir
  Configure-AGSMachine @Args 
}

### EOF
