﻿Param(
    [Parameter(Mandatory=$true)]
    [ValidateNotNullOrEmpty()]
    [string]$OperationId
)

$ConfigXmlFileName = 'Config.xml'

function Log-Message([string]$Message){
    if($Message -ne $null -and $Message.Length -gt 0) {
        $Text = "[$OperationId][$(Get-Date)] $Message"
        Add-Content "$env:SystemDrive\esri\arcgis\db-install-log.txt" $Text -Force
        Write-Output $Text
    }
}
function Ensure-WindowsFeature([string]$WindowsFeatureName, [string]$InstallLogFilePath)
{
    $Check = Get-WindowsFeature -Name $WindowsFeatureName
    if(($Check -eq $null) -or ($Check.Installed -eq $false))
    {
        Log-Message "Installing Windows Feature [$WindowsFeatureName]"
        Install-WindowsFeature -Name $WindowsFeatureName -IncludeAllSubFeature  -IncludeManagementTools -LogPath $InstallLogFilePath | Out-Null
        Log-Message "Finished Installing Windows Feature [$WindowsFeatureName]"
    }
}


function To-HttpBody($props){
    [string]$str = ''
    foreach($prop in $props.Keys){                
        $key = [System.Web.HttpUtility]::UrlEncode($prop)
        $value = [System.Web.HttpUtility]::UrlEncode($props[$prop])
        $str += "$key=$value&"
    }
    if($str.Length -gt 0) {
        $str = $str.Substring(0, $str.Length - 1)
    }
    $str
}

function UploadFile([Uri]$url, [string]$requestUri, [string]$filePath, [string]$fileContentType, $formParams) 
{    
    # Analysis Server expects a signature based only on f=pjson for the uploads API multi part form post       
    $endPoint = $url.AbsoluteUri # + '?f=json&signature=' + [System.Web.HttpUtility]::UrlEncode($global:signature); 
    
    [System.Net.WebRequest]$webRequest = [System.Net.WebRequest]::Create($endPoint);
    $webRequest.ServicePoint.Expect100Continue = $false;
    $webRequest.Method = "POST";    

    $boundary = [System.Guid]::NewGuid().ToString();
    $header = "--{0}" -f $boundary;
    $footer = "--{0}--" -f $boundary;
    $webRequest.ContentType = "multipart/form-data; boundary={0}" -f $boundary;

    [System.IO.Stream]$reqStream = $webRequest.GetRequestStream();    

    $enc = [System.Text.Encoding]::GetEncoding("UTF-8");
    $headerPlusNewLine = $header + [System.Environment]::NewLine;
    [byte[]]$headerBytes = $enc.GetBytes($headerPlusNewLine);
    [void]$reqStream.Write($headerBytes,0, $headerBytes.Length);

    [System.IO.FileInfo]$fileInfo = New-Object "System.IO.FileInfo" -ArgumentList $filePath;   

    #### File Header ####
    $fileHeader = "Content-Disposition: form-data; name=""{0}""; filename=""{1}""" -f "itemFile", $fileInfo.Name;
    $fileHeader = $fileHeader + [System.Environment]::NewLine;    
    [byte[]]$fileHeaderBytes = $enc.GetBytes($fileHeader);
    [void]$reqStream.Write($fileHeaderBytes,0, $fileHeaderBytes.Length);
    
    #### File Content Type ####
    [string]$fileContentTypeStr = "Content-Type: {0}" -f $fileContentType;
    $fileContentTypeStr = $fileContentTypeStr + [System.Environment]::NewLine + [System.Environment]::NewLine;
    [byte[]]$fileContentTypeBytes = $enc.GetBytes($fileContentTypeStr);    
    [void]$reqStream.Write($fileContentTypeBytes,0, $fileContentTypeBytes.Length);    
    
    #### File #####
    [System.IO.FileStream]$fileStream = New-Object 'System.IO.FileStream' -ArgumentList @($filePath, [System.IO.FileMode]::Open, [System.IO.FileAccess]::Read);
    $fileStream.CopyTo($reqStream);
    $fileStream.Flush();
    $fileStream.Close();

    #### Use StreamWrite to write remaining form parameters ####
    [System.IO.StreamWriter]$streamWriter = New-Object 'System.IO.StreamWriter' -ArgumentList $reqStream;
        
    [void]$streamWriter.WriteLine("");
    
    foreach($formParam in $formParams.GetEnumerator()) {
        [void]$streamWriter.WriteLine($header);
        [void]$streamWriter.WriteLine(("Content-Disposition: form-data; name=""{0}""" -f $formParam.Name));
        [void]$streamWriter.WriteLine("");
        [void]$streamWriter.WriteLine($formParam.Value);
    }     
    [void]$streamWriter.WriteLine($footer);
    $streamWriter.Flush();  
     
    $resp = $webRequest.GetResponse()
    $rs = $resp.GetResponseStream()
    [System.IO.StreamReader]$sr = New-Object System.IO.StreamReader -argumentList $rs
    $global:restResponse =  $sr.ReadToEnd() | ConvertFrom-Json
}


function Register-EGDBWithServerSite([string]$SiteUrl, [string]$SDEFilePath, [string]$Server, [string]$Database, [string]$SiteAdminUserName, [string]$SiteAdminPassword)
{
     [System.Reflection.Assembly]::LoadWithPartialName("System.Web") | Out-Null
     [System.Reflection.Assembly]::LoadWithPartialName("System.Net") | Out-Null
     [System.Net.ServicePointManager]::ServerCertificateValidationCallback = {$true}

    [string]$GenerateTokenUrl = $SiteUrl.TrimEnd('/') + '/admin/generateToken'
    Log-Message -Message "Requesting Token at $GenerateTokenUrl"
    $WebParams = @{ username = $SiteAdminUserName
                    password = $SiteAdminPassword                    
                    client = 'requestip'
                    f = 'json'
                  }        
    $HttpRequestBody = To-HttpBody -props $WebParams
    $response = Invoke-RestMethod -Method Post -Uri $GenerateTokenUrl -Body $HttpRequestBody -TimeoutSec 30 
    if($response.status.error -ne $null) {
        throw $response.status.error.messages
    }
    [string]$token = $response.token
    $WebParams = @{ token = $token
                    f = 'json'
                  }

    ###
    ### Check that the system publishing tool is available
    ###
    [string]$PubGPServerUrl = $SiteUrl + '/admin/services/System/PublishingTools.GPServer/status'
    Log-Message -Message "Checking that the system publishing tool is available at $PubGPServerUrl"
    $HttpRequestBody = To-HttpBody -props $WebParams
    $response = Invoke-RestMethod -Method Post -Uri $PubGPServerUrl -Body $HttpRequestBody -TimeoutSec 30 
    if($response.status.error -ne $null) {
        throw "Error checking System Publishing Tool:- $($response.status.error.messages)"
    }
    if($response.configuredState -ne 'STARTED' -or $response.realTimeState -ne 'STARTED') {
        throw "Publishing Tools GP Server not in STARTED State"
    }
    
    [string]$UploadItemUrl = $SiteUrl.TrimEnd('/') + '/admin/uploads/upload'     
    Log-Message -Message "Uploading File to $UploadItemUrl"
    UploadFile -url $UploadItemUrl -requestUri $UploadItemUrl -filePath $SDEFilePath -fileContentType 'application/octet-stream' -formParams $WebParams    
    if($global:restResponse.status.error -ne $null) {
        throw "Error uploading .sde file. Error:- $($global:restResponse.status.error.messages)"
    }
    [string]$ItemId = $global:restResponse.item.itemID
    
    
    ###
    ### Submit a job to to the 'Get Database Connection' GP Tool 
    ###
    [string]$SubmitJobUrl = $SiteUrl.TrimEnd('/') + '/rest/services/System/PublishingTools/GPServer/Get%20Database%20Connection%20String/submitJob'
    Log-Message -Message "Submitting Job to $SubmitJobUrl"
    $WebParams = @{ token = $token
                    f = 'json'
                    in_inputData = $ItemId
                    in_connDataType = 'UPLOADED_CONNECTION_FILE_ID'
                  }
     $HttpRequestBody = To-HttpBody -props $WebParams
     $response = Invoke-RestMethod -Method Post -Uri $SubmitJobUrl -Body $HttpRequestBody -TimeoutSec 30 
     if($response.status.error -ne $null) {
        throw "Error submitting job to 'Get Database Connection' GP Tool $($response.status.error.messages)"
     }

     [string]$JobId = $response.jobId
     [int]$NumAttempts = 0
     [bool]$Done = 0
     [string]$CheckJobStatusUrl = $SiteUrl.TrimEnd('/') + '/rest/services/System/PublishingTools/GPServer/Get%20Database%20Connection%20String/jobs/' + $JobId
     $WebParams = @{ token = $token
                     f = 'json'                   
                  }
     $HttpRequestBody = To-HttpBody -props $WebParams
     [string]$ParamUrl = $null
     while(-not $Done -and $NumAttempts -lt 10) {
        $response = Invoke-RestMethod -Method Post -Uri $CheckJobStatusUrl -Body $HttpRequestBody -TimeoutSec 30 
        Log-Message -Message "Checking Job status at $CheckJobStatusUrl"
        if($response.status.error -ne $null) {
            throw "Error checking job status for job $JobId. Error:- $($response.status.error.messages)"
        }
        if($response.jobStatus -eq 'esriJobSucceeded') {
            $ParamUrl = $response.results.out_connectionString.paramUrl
        }
        else {
            Start-Sleep -Seconds 5
        }
        $NumAttempts++
     }

     [string]$OutParamUrl = $SiteUrl.TrimEnd('/') + '/rest/services/System/PublishingTools/GPServer/Get%20Database%20Connection%20String/jobs/' + "$JobId/$ParamUrl"
     Log-Message -Message "Get Job Result at $OutParamUrl"
     $response = Invoke-RestMethod -Method Post -Uri $OutParamUrl -Body $HttpRequestBody -TimeoutSec 30 
     if($response.status.error -ne $null) {
        throw "Error get job output for job $JobId. Error:- $($response.status.error.messages)"
    }
    ##
    ## Validating Data Item
    ##
    [string]$ConnString = $response.value
    [string]$ValidateDataItemUrl = $SiteUrl.TrimEnd('/') + '/admin/data/validateDataItem'
    $item = @{
                type = 'egdb'
                info = @{
                    dataStoreConnectionType = 'serverOnly'
                    isManaged = $true
                    connectionString = $ConnString
                }
                path = "/enterpriseDatabases/$Server_$Database"
            }
    $WebParams = @{ token = $token
                    f = 'json' 
                    item = ConvertTo-Json $item -Depth 6                  
                  }     
     $HttpRequestBody = To-HttpBody -props $WebParams
     Log-Message -Message "Validating  Item Result at $ValidateDataItemUrl"
     $response = Invoke-RestMethod -Method Post -Uri $ValidateDataItemUrl -Body $HttpRequestBody -TimeoutSec 30 
     if($response.status.error -ne $null) {
        throw "Error getting job output for job $JobId. Error:- $($response.status.error.messages)"
    }

    ##
    ## Registering Data Item
    ##
    [string]$RegisterDataItemUrl = $SiteUrl.TrimEnd('/') + '/admin/data/registerItem'
    Log-Message -Message "Validating  Item Result at $RegisterDataItemUrl"
     $response = Invoke-RestMethod -Method Post -Uri $RegisterDataItemUrl -Body $HttpRequestBody -TimeoutSec 30 
     if($response.status.error -ne $null) {
        throw "Error get job output for job $JobId. Error:- $($response.status.error.messages)"
    }
    Log-Message -Message "Response received from registerItem $($response)"
}


Try
{    
    [string]$BaseDirectory = "$env:SystemDrive\esri\arcgis"
    New-Item -Path $BaseDirectory -ItemType 'directory' -Force -ErrorAction Ignore | Out-Null

    [string]$CurrentExecutionFolder = $(Get-Location).Path
    Log-Message "Script execution started by $env:username in directory $CurrentExecutionFolder. OperationId:- $OperationId"

    ###
    ### Download Location
    ###
    [string]$DownloadLocation = Join-Path $BaseDirectory 'Downloads'
    New-Item -Path $DownloadLocation -ItemType 'directory' -Force -ErrorAction Ignore | Out-Null

    Log-Message "Download Location:- [$DownloadLocation]"  

    if(-not(Test-Path "$CurrentExecutionFolder\$ConfigXmlFileName"))
    {
        throw "Unable to find file $CurrentExecutionFolder\$ConfigXmlFileName"
    }
    
    [xml]$XmlConfig = Get-Content "$CurrentExecutionFolder\$ConfigXmlFileName"

    ###
    ### Install Pre-Requisites
    ###
    [string]$WindowsFeaturesLogs = Join-Path $DownloadLocation 'WindowsFeaturesLogs'   
    $XmlConfig.VirtualMachine.PreRequisites.WindowsFeatures.WindowsFeature.Name | ForEach-Object {
        if($_ -ne $null)
        {
            Log-Message "Installing Pre-requisite Windows Feature [$($_)]"
            Ensure-WindowsFeature $_ -InstallLogFilePath (Join-Path $WindowsFeaturesLogs "$($_)-install.log")
        }
    }

    if($XmlConfig.VirtualMachine.EGDBConfig -ne $null) {     
        [string]$PythonScriptFileName = 'SqlAzure_enable_enterprise_egdb.py'   
        $LicenseFilePath = $ExecutionContext.InvokeCommand.ExpandString("$($XmlConfig.VirtualMachine.EGDBConfig.LicenseFilePath)")
        if(-not (Test-Path $LicenseFilePath)) {
            throw "License file not found at expected location $LicenseFilePath" 
        }
        ## Having a space in the path to the license file causes issue
        ## Copy the file temporarily to root of the system drive
        Copy-Item -Path $LicenseFilePath -Destination (Join-Path $env:SystemDrive 'temp.ecp')
        $LicenseFilePath = (Join-Path $env:SystemDrive 'temp.ecp')
        #Log-Message "Using license file at $LicenseFilePath"
        
        $PythonPath = ((Get-ChildItem -Path "$env:SystemDrive\Python27" -Filter 'python.exe' -Recurse -File) | Select-Object -First 1 -ErrorAction Ignore)
        if($PythonPath -eq $null) {
            throw "Python27 not found on machine. Please install Python"
        }

        [string]$PythonInterpreterPath = $PythonPath.FullName
        [string]$PythonScriptPath = Join-Path $(Get-Location).Path $PythonScriptFileName
        if(Test-Path $PythonScriptPath)
        {
            Log-Message "Enable EGDB Script '$PythonScriptPath' Exists"

            $ServerName = $XmlConfig.VirtualMachine.EGDBConfig.Server
            $DatabaseName = $XmlConfig.VirtualMachine.EGDBConfig.Database
            $DatabaseUserName = $XmlConfig.VirtualMachine.EGDBConfig.UserName
            $DatabaseUserPassword = $XmlConfig.VirtualMachine.EGDBConfig.Password           

            [string]$Arguments = " ""$PythonScriptPath"" -s $ServerName -d $DatabaseName -u $DatabaseUserName -p $DatabaseUserPassword -l $LicenseFilePath "
            Log-Message "[Running Command] $PythonInterpreterPath ""$PythonScriptPath"" -s $ServerName -d $DatabaseName -u $DatabaseUserName -l $LicenseFilePath "
            [string]$StdOutLogFile = [System.IO.Path]::GetTempFileName()
            [string]$StdErrLogFile = [System.IO.Path]::GetTempFileName()
            Start-Process -FilePath $PythonInterpreterPath -ArgumentList $Arguments -RedirectStandardError $StdErrLogFile -RedirectStandardOutput $StdOutLogFile -Wait
            [string]$StdOut = Get-Content $StdOutLogFile
            if($StdOut -ne $null -and $StdOut.Length -gt 0) {
                Log-Message $StdOut
            }
            [string]$StdErr = Get-Content $StdErrLogFile
            if($StdErr -ne $null -and $StdErr.Length -gt 0) {
                Log-Message "[ERROR] $StdErr"
            }
            Remove-Item $StdOutLogFile -Force -ErrorAction Ignore
            Remove-Item $StdErrLogFile -Force -ErrorAction Ignore

            [string]$SDEFile = Join-Path $env:TEMP "$($ServerName)_$($DatabaseName)_$($DatabaseUserName).sde" 
            if(-not (Test-Path $SDEFile)) {
                 throw "SDE File not found at expected location. $SDEFile"
            }        

            if($XmlConfig.VirtualMachine.ServerCredentials -ne $null) {  
                Register-EGDBWithServerSite -SiteUrl 'http://localhost:6080/arcgis' -SDEFilePath $SDEFile `
                                            -Server $ServerName -Database $DatabaseName `
                                            -SiteAdminUserName $XmlConfig.VirtualMachine.ServerCredentials.AdminUserName `
                                            -SiteAdminPassword $XmlConfig.VirtualMachine.ServerCredentials.AdminPassword
            }
        }
        else
        {
            Log-Message "[ERROR] $PythonScriptPath not found"
        }

        ##
        ## Remove License File 
        ##
        Remove-Item $LicenseFilePath -ErrorAction Ignore

        
        
        ##
        ## Remove .sde file
        ##
        if($SDEFile -ne $null -and $SDEFile.Length -gt 0) {
            Remove-Item $SDEFile -ErrorAction Ignore
        }
    }    
}
Catch
{        
        Log-Message  "[EXCEPTION] $($_)"
}
Finally
{
    ##
    ## Remove XML File (which has confidential information)
    ##
    #Remove-Item "$CurrentExecutionFolder\$ConfigXmlFileName" -ErrorAction Ignore
   

    Log-Message  "Script execution ended at $(Get-Date)"
    Log-Message 'DONE'  
}