﻿param(
    [Parameter(Mandatory=$true, HelpMessage='Name of your Azure Subscription')]
    [ValidateNotNullOrEmpty()]
    [Alias("sub")]
    [string]$SubscriptionName,
        
    [Parameter(Mandatory=$true, HelpMessage='Absolute Path to your Azure Publish Settings File')]
    [ValidateNotNullOrEmpty()]
    [Alias("pub")]
    [string]$PublishSettingsFile,

    [Parameter(Mandatory=$true, HelpMessage='Absolute Path to the location of your ArcGIS Server Installer (Setup.exe)')]
    [ValidateNotNullOrEmpty()]
    [Alias("ags")]
    [string]$AGSSetupPath,

    [Parameter(Mandatory=$true, HelpMessage='Absolute Path to the location of your ArcGIS Server License (license.prvc)')]
    [ValidateNotNullOrEmpty()]
    [Alias("lic")]
    [string]$AGSLicensePath,

    [Parameter(Mandatory=$true, HelpMessage='Storage Account to use for the image generated')]
    [ValidateNotNullOrEmpty()]
    [Alias("sto")]
    [string]$StorageAccountName,

    [Parameter(Mandatory=$false, HelpMessage='Windows Server Stock Image')]
    [string]$StockImageName,       # either Name or Label shoudl be provided, if both  
    [string]$StockImageLabel,      # are missing, the latest image will be assigned automatically 

    [Parameter(Mandatory=$true, HelpMessage='Azure DataCenter Region')]
    [ValidateNotNullOrEmpty()]
    [ValidateSet("East Asia","Southeast Asia","North Europe","West Europe","East US","North Central US","South Central US","West US","Japan East","Japan West","Brazil South")]
    [Alias("CloudServiceLocation","loc")]
    [string]$Location,

    [Parameter(Mandatory=$true, HelpMessage='Name of the output image')]
    [ValidateNotNullOrEmpty()]
    [Alias("resimg","res")]
    [string]$ResultImageName,
    
    
    [Parameter(Mandatory=$false, HelpMessage='Label of the output image')]
    [string]$ResultImageLabel = “$ResultImageName label”,

    [Parameter(Mandatory=$false, HelpMessage='Full path to a log file.')]
    [Alias("log")]
    [string]$LogFile,

    [Parameter(Mandatory=$false, HelpMessage='DEBUG: skip cleaning alocated resources.')]
    [switch]$DontClean
  )

if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

$curDir = Split-Path ((Get-Variable MyInvocation -Scope 0).Value.MyCommand.Path)
. "$curDir\agsCustomScriptExtensionTools.ps1"
. "$curDir\agsAzureStorageTools.ps1"
. "$curDir\agsAzureVMTools.ps1"
Set-Location $curDir

if (!$LogFile) {
  $LogFile = Join-Path (Split-Path $pwd) -ChildPath ("logs\Create-Image-" + (Get-Date -Format "dd-MMM-yyyy-THH-mm-ss") + ".log")
} elseif ($LogFile -ieq "n" -or $LogFile -ieq "no" -or $LogFile -ieq "none" -or $LogFile -ieq "off") {
  $LogFile = $null
}

$VMAdminAccountName       = "a" + (([System.Guid]::NewGuid()).Guid).Replace('-','').Substring(0,19)
$VMAdminAccountPassword   = ([System.Guid]::NewGuid()).Guid

#########################################################################################
##
#. "$curDir\S_DEBUG.ps1"

#########################################################################################
##
try  
{
  Log-Init $LogFile

  Test-PSVersion    "3.0"       # minimum required versions
  Test-AzureVersion "0.8.2"

  log "Checking {0} input parameters" -p ($MyInvocation.MyCommand.Name)
  Test-AGSSetupPath   $AgsSetupPath
  Test-AgsLicenseFile $AgsLicensePath

  Connect-To-Azure $SubscriptionName $PublishSettingsFile $StorageAccountName
  
  Test-CloudServiceLocation $Location                        
  $image = Get-AzureImage -Name $StockImageName -Label $StockImageLabel 

  # checking proposed resulting image name 
  if (Test-AzureImage -Name $ResultImageName) {
    throw (fls "Image {0} already exists. Please choose a different name." -p $ResultImageName)
  }
  
  # TODO: figure out operation ID
  $rnd = Create-RandomString -UnStick -Stick   ## temporary Cloud Service name
  $csn = "cs-$rnd"                             ## Cloud Service name must be globally unique
  $vmn = "vm-$rnd".Substring(0,15)             ## temporary Virtual Machine name (15 char max!)
  
  # sql client lib
  $sqlcli = Join-Path $pwd -ChildPath "SQLClientLib\2012SP1x64\sqlncli.msi"
  if (!(Test-Path $sqlcli)) {
    throw (fls "File not found : {0}" -p $sqlcli)
  }

  # create VM from original stock image and install proper AGS components
  Create-AGSVM -StorageAccountName $StorageAccountName `
               -VMAdminName $VMAdminAccountName -VMAdminPassword $VMAdminAccountPassword `
               -CloudServiceName $csn -CloudServiceLocation $Location `
               -VMachineName $vmn  `
               -ImageName $image.ImageName `
               -AGSSetupPath $AgsSetupPath `
               -AGSLicensePath $AgsLicensePath `
               -SQLClientPath $sqlcli `
               -Firewall "ON" `
               -Sysprep `
               -Wait -DontClean:$DontClean

  log "creating the VM image : name {0}, label {1}" -p $ResultImageName,$ResultImageLabel -fc Yellow
  Save-AzureVMImage -ServiceName $csn -Name $vmn -ImageName $ResultImageName -ImageLabel $ResultImageLabel -Verbose  *>&1 | logr
  log "image created : {0}" -p $ResultImageName -ForegroundColor Green
  log "Create-Image.ps1 : SUCCEEDED" -fc Green
}
catch
{
  $err = log "Create-Image.ps1 : FAILED" -fc Red
  $_ | logr 
  throw $err
}
finally  ### CLEANING 
{
  if (!$DontClean) {
    ## removing temporary cloud service with all related vhds
    if ($csn -and (Test-AzureName -Service -Name $csn)) 
    {
      log "removing temporary cloud service : {0}" -p  $csn
      Remove-AzureService -ServiceName $csn -DeleteAll -Force -verbose

      log "removing blobs related to the service that may still exist in vhds container"
      $sth = New-AzureStorageHelper $StorageAccountName    
      $sth.Connect()
      $sth.OpenContainer("vhds")
      $sth.RemoveBlobs("$csn*",$true)  # Force
      $sth = $null
    }
  }
}

### EOF