﻿Param(
    [Parameter(Mandatory=$true, HelpMessage='Name of your Azure Subscription')]
    [ValidateNotNullOrEmpty()]
    [string]$SubscriptionName,

    [Parameter(Mandatory=$true, HelpMessage='Absolute Path to your Azure Publish Settings File')]
    [ValidateNotNullOrEmpty()]
    [string]$PublishSettingsFile,

    [Parameter(Mandatory=$true, HelpMessage='DNS Name for your ArcGIS Server Site on Azure''s cloudapp.net domain')]
    [ValidateNotNullOrEmpty()]
    [string]$CloudServiceName,

    [Parameter(Mandatory=$true, HelpMessage='Storage Account to use for the virtual machine disk')]
    [ValidateNotNullOrEmpty()]
    [string]$StorageAccountName,

    [Parameter(Mandatory=$true, HelpMessage='Base Image containing ArcGIS Server')]
    [ValidateNotNullOrEmpty()]
    [string]$ImageName,

    [Parameter(Mandatory=$true, HelpMessage='Azure DataCenter Region')]
    [ValidateNotNullOrEmpty()]
    [ValidateSet("East Asia","Southeast Asia","North Europe","West Europe","East US","North Central US","South Central US","West US","Japan East","Japan West","Brazil South")]
    [Alias("CloudServiceLocation","loc")]
    [string]$Location,

    [Parameter(Mandatory=$true, HelpMessage='The new Virtual Machine name. Lower case, 15 characters length limit.')]
    [Alias("VMName","vmn")][string]$VMachineName,  

    [Parameter(Mandatory=$false, HelpMessage='Machine Time Zone')]
    [Alias("TZone","TimeZone","tz")][string]$VMachineTimeZone = "Pacific Standard Time",

    [Parameter(Mandatory=$false, HelpMessage='Machine Size')]
    [Alias("InstanceSize")][string]$VMachineSize = "Medium",

    [Parameter(Mandatory=$true, HelpMessage='VM Admin account name')]
    [ValidateNotNullOrEmpty()]
    [Alias("admin","vmacc")]
    [string]$VMAdminAccountName,

    [Parameter(Mandatory=$true, HelpMessage='VM Admin account password')]
    [ValidateNotNullOrEmpty()]
    [Alias("pswd","vmpswd")]
    [string]$VMAdminAccountPassword,

    [Parameter(Mandatory=$true, HelpMessage='AGS Site Admin account name')]
    [ValidateNotNullOrEmpty()]
    [Alias("sadmin","agsacc")]
    [string]$AGSSiteAdminAccountName,

    [Parameter(Mandatory=$true, HelpMessage='AGS Site Admin account password')]
    [ValidateNotNullOrEmpty()]
    [Alias("spswd","agspswd")]
    [string]$AGSSiteAdminAccountPassword,
    
    [Parameter(Mandatory=$false, HelpMessage='Full path to a log file.')]
    [Alias("log")]
    [string]$LogFile

  )

if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

$curDir = Split-Path ((Get-Variable MyInvocation -Scope 0).Value.MyCommand.Path)
. "$curDir\agsAdminRestTools.ps1"
. "$curDir\agsCustomScriptExtensionTools.ps1"
. "$curDir\agsAzureStorageTools.ps1"
. "$curDir\agsAzureVMTools.ps1"
Set-Location $curDir

if (!$LogFile) {
  $LogFile = Join-Path (Split-Path $pwd) -ChildPath ("logs\Deploy-Site-" + (Get-Date -Format "dd-MMM-yyyy-THH-mm-ss") + ".log")
} elseif ($LogFile -ieq "n" -or $LogFile -ieq "no" -or $LogFile -ieq "none" -or $LogFile -ieq "off") {
  $LogFile = $null
}

#########################################################################################
##
#. "$curDir\S_DEBUG.ps1"

#########################################################################################
##
try  
{
  Log-Init $LogFile

  Test-PSVersion    "3.0"       # minimum required versions
  Test-AzureVersion "0.8.2"

  Connect-To-Azure $SubscriptionName $PublishSettingsFile $StorageAccountName

  ## TODO: add check if the service is empty??
  if (Test-AzureName -Service $CloudServiceName){
    throw (fls "Service {0} is already in use. Please choose an alternative" -p $CloudServiceName)
  }

  # test location
  Test-CloudServiceLocation $Location

  # create new vm
  Create-AGSVM -SubscriptionName $SubscriptionName -PublishSettingsFile $PublishSettingsFile -StorageAccountName $StorageAccountName `
               -CloudServiceName $CloudServiceName -CloudServiceLocation $Location `
               -VMachineName $VMachineName -VMachineTimeZone $VMachineTimeZone -VMachineSize $VMachineSize `
               -VMAdminName $VMAdminAccountName `
               -VMAdminPassword $VMAdminAccountPassword  `
               -ImageName $ImageName `
               -AGSEndpoints

  # Wait for site to come up
  $baseHostUrl = "http://$CloudServiceName.cloudapp.net/"
  log "waiting for Web Server to start up : {0}" -p $baseHostUrl
  while (!(Test-AGSServer $baseHostUrl)) {    
    Sleep -Seconds 3
  }

  log "Web Server is started" -fc Green

  log "Creating ArcGIS Server Site : {0}" -p $baseHostUrl
  try {
    $jobID = Create-AGSSite -ServerURL $baseHostUrl -UserName $AGSSiteAdminAccountName -Password $AGSSiteAdminAccountPassword -RunAsync
  } catch {
    # Sometimes it fails, but site gets created anyway
  }
  Sleep -Seconds 30
  log "ArcGIS Server Site created." -fc Green

  $agsManagerURL = $baseHostUrl.TrimEnd("/") + "/arcgis/manager"
  log "Launching ArcGIS Server Manager : {0}" -p $agsManagerURL
  Start-Process $agsManagerURL 

  log "Deploy-Site.ps1 : SUCCEEDED" -fc Green
}
catch
{
  $err = log "Deploy-Site.ps1 : FAILED" -fc Red
  $_ | logr 
#  throw $err
}

### EOF