﻿Param(
    [Parameter(Mandatory=$true, HelpMessage='Name of your Azure Subscription')]
    [ValidateNotNullOrEmpty()]
    [string]$SubscriptionName,
      
    [Parameter(Mandatory=$true, HelpMessage='Absolute Path to your Azure Publish Settings File')]    
    [string]$PublishSettingsFile,
    
    [Parameter(Mandatory=$true, HelpMessage='Storage Account to use for Virtual Machine Disk')]    
    [string]$StorageAccountName,

    [Parameter(Mandatory=$true, HelpMessage='DNS Name for your Service for ArcGIS Site on Azure''s cloudapp.net domain')]    
    [string]$CloudServiceName,

    [Parameter(Mandatory=$false, HelpMessage='Virtual Machine Instance Name')]    
    [string]$VirtualMachineName,   
      
    [Parameter(Mandatory=$true, HelpMessage='UserName for the ArcGIS Server Primary Site Administrator Account')]   
    [string]$SiteAdminUserName,
    
    [Parameter(Mandatory=$true, HelpMessage='Password for the ArcGIS Server Primary Site Administrator Account')]    
    [string]$SiteAdminPassword,
    
    [Parameter(Mandatory=$true, HelpMessage='SQL DB Server Name. Provide an empty string to have the system create a new Server for you')]
    [AllowNull()]
    [AllowEmptyString()]
    [string]$DatabaseServerName,

    [Parameter(Mandatory=$true, HelpMessage='SQL DB Name. Provide the name of an existing database or the name of new database that will be created for you')]    
    [AllowNull()]
    [AllowEmptyString()]
    [string]$DatabaseName,

    [Parameter(Mandatory=$true, HelpMessage='SQL DB Server Administrator User Name')]
    [string]$DatabaseServerAdminUserName,

    [Parameter(Mandatory=$true, HelpMessage='SQL DB Server Administrator Password')]    
    [string]$DatabaseServerAdminPassword,

    [Parameter(Mandatory=$true, HelpMessage='Azure Data Center Region')]
    [string]$Location,

    [Parameter(Mandatory=$false, HelpMessage='Full Path to the Operation Log File')]
    [AllowNull()]
    [AllowEmptyString()]
    [string]$LogFilePath
 )

 . .\Utils.ps1
 . .\InputUtils.ps1

 if($PublishSettingsFile -eq $null -or $PublishSettingsFile.Length -lt 1) {
    $PublishSettingsFile = Prompt-FileLocation -PromptMessage 'Browse to location of Publish Settings file' -Filter 'Publish Settings Files(*.publishsettings,*.xml)|*.publishsettings;.xml'
    if($PublishSettingsFile -eq $null -or $PublishSettingsFile.Length -lt 1) {
        Write-Host 'Cancelled by user request' -ForegroundColor Yellow
        return
    }
 }else{
    if(-not (Test-Path $PublishSettingsFile)) {
        throw "License File not found at $PublishSettingsFile"
    }    
 }

 
   
function Log-Message([string]$Message)
{
     if($Message -ne $null -and $Message.Length -gt 0) {
        $Text = "[$(Get-Date)] $Message"        
        Write-Output $Text
        if($LogFilePath -ne $null -and $LogFilePath.Length -gt 0) {
            Add-Content $LogFilePath -Value $Text
        }
    }
}

function Create-XmlFile([string]$VirtualMachineName, [string]$OutputFilePath, $EGDBConfig, $ServerCredentials, [string]$PreRequisiteWindowsFeatures)
{
    $xmlWriter = New-Object System.XMl.XmlTextWriter($OutputFilePath,$Null)
   
    $xmlWriter.Formatting = 'Indented'
    $xmlWriter.Indentation = 1
    $XmlWriter.IndentChar = "`t"
 
    # write the header
    $xmlWriter.WriteStartDocument()
 
    $xmlWriter.WriteStartElement('VirtualMachine')
    $xmlWriter.WriteAttributeString('version', '1.0')    
    $xmlWriter.WriteAttributeString('name', $VirtualMachineName)    
        
    if($PreRequisiteWindowsFeatures -ne $Null)
    {
        $xmlWriter.WriteStartElement('PreRequisites')
        $xmlWriter.WriteStartElement('WindowsFeatures')
        foreach($PreRequisiteWindowsFeature in $PreRequisiteWindowsFeatures)
        {
            $xmlWriter.WriteStartElement('WindowsFeature')
            $xmlWriter.WriteElementString('Name',$PreRequisiteWindowsFeature)
            $xmlWriter.WriteEndElement()
        }
        $xmlWriter.WriteEndElement()
        $xmlWriter.WriteEndElement()
    }            

    if($EGDBConfig -ne $Null) 
    {
        $xmlWriter.WriteStartElement('EGDBConfig')
        $xmlWriter.WriteElementString('Server',$EGDBConfig.Server)
        $xmlWriter.WriteElementString('Database',$EGDBConfig.Database)
        $xmlWriter.WriteElementString('UserName',$EGDBConfig.UserName)
        $xmlWriter.WriteElementString('Password',$EGDBConfig.Password) 
        $xmlWriter.WriteElementString('LicenseFilePath',$EGDBConfig.LicenseFilePath)    
        $xmlWriter.WriteEndElement() # EGDBConfig
    }

    if($ServerCredentials -ne $Null) 
    {
        $xmlWriter.WriteStartElement('ServerCredentials')
        $xmlWriter.WriteElementString('AdminUserName',$ServerCredentials.UserName)
        $xmlWriter.WriteElementString('AdminPassword',$ServerCredentials.Password)
        $xmlWriter.WriteEndElement()
    }

    $xmlWriter.WriteEndElement() # Virtual Machine

    $xmlWriter.WriteEndDocument()
    $xmlWriter.Flush()
    $xmlWriter.Close()
}

[string]$Guid        = [System.Guid]::NewGuid().ToString('N')
[string]$ConfigXmlFileName   = 'Config.xml'
[string]$BootStrapScript = 'Configure-Database.ps1'  
[string]$PythonScriptFileName = 'SqlAzure_enable_enterprise_egdb.py' 
[string]$CurrentExecutionFolder = $(Get-Location).Path

if($LogFilePath -eq $null -or $LogFilePath.Length -lt 1) {    
    $LogFilePath = Join-Path $(Split-Path $CurrentExecutionFolder -Parent) "logs"
    New-Item $LogFilePath -ItemType directory -Force -ErrorAction Ignore | Out-Null #Ensure Folder exists
    $LogFilePath = Join-Path $LogFilePath "Register-EGDB-$(Get-Date -Format "dd-MMM-yyyy-THH-mm-ss").log"
 }
 Write-Output "Logging to $LogFilePath"

try
{
    Connect-ToAzure -PublishSettingsFile $PublishSettingsFile -SubscriptionName $SubscriptionName -StorageAccountName $StorageAccountName
    if($StorageAccountName -eq $null -or $StorageAccountName.Length -lt 1) {
        $StorageAccountNames = @()
        Get-AzureStorageAccount | ForEach-Object { $StorageAccountNames += $_.StorageAccountName }
        $StorageAccountName = Prompt-StringInput -PromptMessage 'Enter Storage Account' -ValidateNotNullOrEmpty -DomainOfInputValues $StorageAccountNames -InputRequestedMessage 'Input Required'
    }
    $StorageContext = Create-StorageAccountContext -StorageAccountName $StorageAccountName
  
    ###
    ### Ensure Database Server 
    ###
    if($DatabaseServerName -eq $null -or $DatabaseServerName.Length -lt 1) {
        Log-Message 'SQL Database Server information not supplied by user. Prompting for input.'
        $AvailableServers = @()
        Get-AzureSqlDatabaseServer | ForEach-Object { $AvailableServers += $_.ServerName } 
        $DatabaseServerName = Prompt-StringInput -PromptMessage 'Select Existing Database Server Name (Leave empty to create a new Server)' -DomainOfInputValues $AvailableServers 

        if($DatabaseServerName -eq $null -or $DatabaseServerName.Length -lt 1) {
            ## User didn't pick an available Server. Wants a new one
            ###
            ### Prompt user for Location of Database Server
            ###
            if($Location -eq $null -or $Location.Length -lt 1) {            
                [string[]]$AllRegions = Get-AvailableAzureLocations
                $Location = Prompt-StringInput -PromptMessage 'Enter Azure Data Center Location for SQL Database Server' -ValidateNotNullOrEmpty -DomainOfInputValues $AllRegions -InputRequestedMessage 'Input Required'
            }
        
            if($DatabaseServerAdminUserName -eq $null -or $DatabaseServerAdminPassword -eq $null -or $DatabaseServerAdminUserName.Length -lt 1 -or $DatabaseServerAdminPassword.Length -lt 1){
                $ServerAdminCredentials = Prompt-Credentials -PromptMessage 'Enter Database Server Administrator Credentials' -UserName $DatabaseServerAdminUserName -ValidateNotNullOrEmpty -Confirm  `
                                                                -ConfirmPromptMessage 'Confirm Database Server Administrator Credentials' 
                if($ServerAdminCredentials -eq $null) {
                    Write-Host 'Cancelled by user request' -ForegroundColor Yellow
                    return
                }
                $DatabaseServerAdminUserName = $ServerAdminCredentials.GetNetworkCredential().UserName
                $DatabaseServerAdminPassword = $ServerAdminCredentials.GetNetworkCredential().Password
            } 
       
            Log-Message "Creating New-AzureSqlDatabaseServer in location $Location with Administrator Login $DatabaseServerAdminUserName"
            $Server = New-AzureSqlDatabaseServer -AdministratorLogin $DatabaseServerAdminUserName -AdministratorLoginPassword $DatabaseServerAdminPassword -Location $Location -Confirm 
            New-AzureSqlDatabaseServerFirewallRule -ServerName $Server.ServerName -AllowAllAzureServices -RuleName 'AllowAzureServices' -Force | Out-Null # Allow Azure Services            
            $DatabaseServerName = $Server.ServerName   
            
         }
    }
    ##
    ## Validate Database Server
    ##
    $Server = Get-AzureSqlDatabaseServer -ServerName $DatabaseServerName -ErrorAction Ignore
    if($Server -eq $null) {
        throw "SQL Database Server $DatabaseServerName not found."
    }
    else {
        if($DatabaseServerAdminUserName -eq $null -or $DatabaseServerAdminPassword -eq $null -or $DatabaseServerAdminUserName.Length -lt 1 -or $DatabaseServerAdminPassword.Length -lt 1){
            $DatabaseServerAdminUserName = $Server.AdministratorLogin
            $ServerAdminCredentials = Prompt-Credentials -PromptMessage 'Enter Database Server Administrator Credentials' -UserName $DatabaseServerAdminUserName -ValidateNotNullOrEmpty -Confirm  `
                                                            -ConfirmPromptMessage 'Confirm Database Server Administrator Credentials' 
            if($ServerAdminCredentials -eq $null) {
                Write-Host 'Cancelled by user request' -ForegroundColor Yellow
                return
            }
            $DatabaseServerAdminUserName = $ServerAdminCredentials.GetNetworkCredential().UserName
            $DatabaseServerAdminPassword = $ServerAdminCredentials.GetNetworkCredential().Password
        } 
    }

    ###
    ### Ensure Database  
    ###
    if($DatabaseName -eq $null -or $DatabaseName.Length -lt 1) {
        ###
        ### Prompt user for Database Name
        ###
        $DatabaseNames = @()
        Get-AzureSqlDatabase -ServerName $DatabaseServerName | ForEach-Object { if($_.Name -ne 'master') { $DatabaseNames += $_.Name } }
        $DatabaseName = Prompt-StringInput -PromptMessage 'Enter SQL Database Name' -ValidateNotNullOrEmpty -InputRequestedMessage 'Input Required' -DomainOfInputValues $DatabaseNames     
    }
    $Database = Get-AzureSqlDatabase -ServerName $DatabaseServerName -DatabaseName $DatabaseName -ErrorAction Ignore
    if($Database -eq $null) {
        New-AzureSqlDatabase -ServerName $DatabaseServerName -DatabaseName $DatabaseName -Edition Business 
    }
    

    ###
    ### Get/Validate that CloudService Exists
    ###
    if($CloudServiceName -eq $null -or $CloudServiceName.Length -lt 1) {
        $CloudServiceNames = @()
        Get-AzureService | ForEach-Object { $CloudServiceNames += $_.ServiceName }
        $CloudServiceName = Prompt-StringInput -PromptMessage 'Choose Cloud Service' -ValidateNotNullOrEmpty -DomainOfInputValues $CloudServiceNames -InputRequestedMessage 'Input Required'
    }
    if(-not (Test-AzureName -Service -Name $CloudServiceName)) {
        throw "CloudService $CloudServiceName does not exists"
    }

    ###
    ### Get/Validate that Virtual Machine Exists
    ###
    $VirtualMachine = $null
    if($VirtualMachineName -eq $null -or $VirtualMachineName.Length -lt 1) {
        $VirtualMachine = (Get-AzureVM -ServiceName $CloudServiceName)[0]
    }
    else {
        $VirtualMachine = Get-AzureVM -ServiceName $CloudServiceName -Name $VirtualMachineName
    }
    if($VirtualMachine -eq $null) {
        throw "Virtual Machine $VirtualMachineName does not exist"
    } 

    $ContainerName = "esri-wksp-$Guid"    
    $BlobContainer = Get-AzureStorageContainer -Name $ContainerName -Context $StorageContext -ErrorAction Ignore
    if($null -eq $BlobContainer)
    {
        New-AzureStorageContainer -Name $ContainerName -Context $StorageContext | Out-Null
        $BlobContainer = Get-AzureStorageContainer -Name $ContainerName -Context $StorageContext -ErrorAction Stop
    }    
  
    ##
    ## Upload BootStrap Script Files, XML Config File, LicenseFile and Python Script
    ##
    $BootStrapScriptFilePath = Join-Path $CurrentExecutionFolder $BootStrapScript
    Set-AzureStorageBlobContent -File $BootStrapScriptFilePath -Container $ContainerName -Context $StorageContext -Force | Out-Null
    $PythonScriptFilePath = Join-Path $CurrentExecutionFolder $PythonScriptFileName
    Set-AzureStorageBlobContent -File $PythonScriptFilePath -Container $ContainerName -Context $StorageContext -Force | Out-Null
          
    $XmlFilePath = Join-Path $([System.IO.Path]::GetTempPath()) "Config$Guid.xml"
    if(-not $DatabaseServerName.EndsWith('.database.windows.net'))
    {
        $DatabaseServerName = $DatabaseServerName + '.database.windows.net'
    }
    $EGDBConfig = @{
        Server = $DatabaseServerName
        Database = $DatabaseName
        UserName = $DatabaseServerAdminUserName
        Password = $DatabaseServerAdminPassword
        LicenseFilePath = '$env:SystemDrive\Program Files\ESRI\License10.2\sysgen\keycodes'
    }
    if($SiteAdminUserName -eq $null -or $SiteAdminUserName.Length -lt 1 -or $SiteAdminPassword -eq $null -or $SiteAdminPassword.Length -lt 1) {
        $SiteAdminUserName = 'siteadmin' # Default
        $SiteAdminCredentials = Prompt-Credentials -PromptMessage 'Enter Site Admin Credentials' -ValidateNotNullOrEmpty -Confirm `
                                                -ConfirmPromptMessage 'Confirm Site Admin Credentials' -UserName $SiteAdminUserName
        if($SiteAdminCredentials -eq $null) {
            Write-Host 'Cancelled by user request' -ForegroundColor Yellow
            return
        }
        $SiteAdminUserName = $SiteAdminCredentials.GetNetworkCredential().UserName
        $SiteAdminPassword = $SiteAdminCredentials.GetNetworkCredential().Password
    }
    $ServerCreds = @{
        UserName = $SiteAdminUserName
        Password = $SiteAdminPassword
    }
    Create-XmlFile -OutputFilePath $XmlFilePath -VirtualMachineName $VirtualMachineName -EGDBConfig $EGDBConfig -ServerCredentials $ServerCreds `
                    -PreRequisiteWindowsFeatures @('Web-Net-Ext')
    Set-AzureStorageBlobContent -File $XmlFilePath -Container $ContainerName -Context $StorageContext -Blob $ConfigXmlFileName -Force | Out-Null

    [string]$OperationId = $Guid
       

    ##
    ## Run the Remote script
    ##
    Set-AzureVMCustomScriptExtension -ContainerName $ContainerName -FileName @($BootStrapScript, $ConfigXmlFileName, $PythonScriptFileName) `
                                        -Run $BootStrapScript -VM $VirtualMachine -Argument " -OperationId $OperationId " `
                                        | Update-AzureVM 

    ##
    ## Wait for Script to complete
    ##
    [bool]$Success = $false
    [bool]$Done = $false    
    [int]$TotalSeconds = 60 * 15 # 15 minutes
    [int]$TimeConsumedInSeconds = 0
    [int]$SleepTimeInSeconds = 30
    [string]$LastLineText = $null
    [string]$OperationPrefix = "[$OperationId]"
    while((-not $Done) -and ($TimeConsumedInSeconds -lt $TotalSeconds))
    {    
        $VirtualMachine = Get-AzureVM -ServiceName $CloudServiceName -Name $VirtualMachineName    
    
        $Extension = ($VirtualMachine.ResourceExtensionStatusList | Where-Object { $_.HandlerName -eq 'Microsoft.Compute.CustomScriptExtension' })
      
        $StdOut = $Extension.ExtensionSettingStatus.SubStatusList | Where-Object { $_.Name-eq 'StdOut' }
        $StdErr = $Extension.ExtensionSettingStatus.SubStatusList | Where-Object { $_.Name -eq 'StdErr' }

        if($StdOut -ne $null -and $StdOut.FormattedMessage -ne $null)
        {
            [string]$StdOutMessage = $StdOut.FormattedMessage.Message
            if($StdOutMessage -ne $null -and $StdOutMessage.Length -gt 0)
            {
                [string[]]$RawLines = $StdOutMessage -split '\\n' 
                $Lines = @()
                foreach($RawLine in $RawLines)
                {
                    if($RawLine -ne $null -and $RawLine.Length -gt 0) {
                        $Lines += $RawLine                        
                    }
                }
                [int]$Index = $Lines.Count -1
                if($LastLineText -ne $null)
                {
                    while($Index -gt 0 -and $LastLineText -ne $Lines[$Index-1])
                    {
                        $Index--
                    }
                }
                else
                {
                    $Index = 0
                }            
                while($Index -lt $Lines.Count)
                {
                    [string]$Line = $Lines[$Index]
                    if($Line -ne $null -and $Line.Length -gt 0 -and $Line.StartsWith($OperationPrefix))
                    {
                        Log-Message $Line.Substring($OperationPrefix.Length)
                        $LastLineText = $Line.TrimEnd()                
                    }
                    $Index++
                }
                if($LastLineText -ne $null -and $LastLineText.EndsWith('DONE') -and $Line.StartsWith($OperationPrefix))
                {
                    $Done = $true
                    $Success = $true
                }
            }
        }

        if($StdErr -ne $null -and $StdErr.FormattedMessage -ne $null)
        {
            [string]$StdErrMessage = $StdErr.FormattedMessage.Message
            if($StdErrMessage -ne $null -and $StdErrMessage.Length -gt 0)
            {
                Log-Message $StdErrMessage
                Write-Error $StdErrMessage
                $Done = $true
            }
        }

        Write-Progress -Activity 'Registering Enterprise GeoDatabase on Virtual Machine' -Status 'Registering Enterprise GeoDatabase on ArcGIS Server' -PercentComplete (($TimeConsumedInSeconds/$TotalSeconds)*100)
        Start-Sleep -Seconds $SleepTimeInSeconds
        $TimeConsumedInSeconds += $SleepTimeInSeconds
    }
    Write-Progress -Activity 'Registering Enterprise GeoDatabase on Virtual Machine' -Status "SQL Azure Database registered as Enterprise GeoDatabase for ArcGIS Server Site created at http://$CloudServiceName.cloudapp.net/arcgis/home" -Completed    

    if($Success -eq $true)
    {
        [string]$ServerUrl = "http://$CloudServiceName.cloudapp.net/arcgis/manager"
        Start-Process $ServerUrl

        ##
        ## To Allow PowerShell Pipeline for composite workflows, return the URL to the deployed Server site
        ##
        return "http://$CloudServiceName.cloudapp.net/"
    }
    else
    {
        Write-Error 'Registeration of Enterprise GeoDatabase on Virtual Machine failed to complete in expected fashion'
    }
}
finally
{
    ##
    ## Cleanup
    ##
    if($ContainerName -ne $null -and $StorageContext -ne $null) {
        Log-Message "Removing Workspace Container $ContainerName in Blob Storage"
        Remove-AzureStorageContainer -Name $ContainerName -Context $StorageContext -Force -ErrorAction Ignore | Out-Null       
    }
    if($XmlFilePath -ne $null) {
         Remove-Item -Path $XmlFilePath -Force -ErrorAction Ignore | Out-Null
    }
}