﻿if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

#################################################################################
## 
##  ArcGIS Server Admin REST operations
##
#################################################################################
##
function Get-AGSSiteInfo([string]$ServerURL="http://localhost:6080",[int]$TimeOut=30)
{
  $cmdUrl  = $ServerURL.TrimEnd("/") + "/arcgis/rest/info"
  $cmdBody = "f=json"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing |
  Select-Object StatusCode,StatusDescription,@{Name="Response"; Expression = {$_.Content | ConvertFrom-Json}}
}

#################################################################################
##
function Test-AGSServer([string]$ServerURL="http://localhost:6080")
{
  $cmdUrl  = $ServerURL.TrimEnd("/") + "/arcgis/admin"
  try{  
    $res = Invoke-WebRequest -Uri $cmdUrl -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing | Select-Object StatusCode   
    if ($res -and ($res.StatusCode -eq 200)) {
      return $true
    }
  } catch { 
    ## does not matter
  }

  return $false
}

#################################################################################
##
function Get-AGSAdminToken([string]$ServerURL="http://localhost:6080",$UserName,$Password,$Expiration=1000) 
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/generateToken"
  $cmdBody = "f=json&username=$UserName&password=$Password&client=requestip&expiration=$Expiration"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing 
  if ($res) {
    return ($res.Content | ConvertFrom-Json)  # PSObject with 2 props : 'token' and 'expires'
  }
}

#################################################################################
## 
function Check-ResponseStatus($Response)
{
  $parentFunc = (Get-Variable MyInvocation -Scope 1).Value.MyCommand.Name

  if (!$Response) { 
    throw (fls "ERROR: {0} response is NULL" -p $parentFunc)
  }
  if ($Response.status -and ($Response.status -ieq "error")) { 
    throw (fls "ERROR: {0} failed. {1}" -p $parentFunc,($Response.messages -join " "))
  }
}   

#################################################################################
## 
function ConvertTo-HtmlBody([hashtable]$Props)
{
  $arr = $Props.GetEnumerator() | %{'{0}={1}' -f ([System.Web.HttpUtility]::UrlEncode($_.Name)),([System.Web.HttpUtility]::UrlEncode($_.Value))}
  $arr -join '&'
}

#################################################################################
##
function Delete-AGSSite([string]$ServerURL="http://localhost:6080",[string]$AdminToken,[int]$TimeOut=300) 
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/deleteSite"
  $cmdBody = "f=json&token=$AdminToken"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing -TimeoutSec $TimeOut
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response 
}

#################################################################################
##
function Get-AGSJobStatus([string]$ServerURL="http://localhost:6080",[string]$JobID,[string]$AdminToken,[int]$TimeOut=30) 
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/system/jobs/"  +  $JobID
  $cmdBody = "f=json&token=$AdminToken"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing -TimeoutSec $TimeOut
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response 
}

<# -----------  to get the job status  -------------
  try {
  $token = Get-AGSAdminToken $ServerURL  -UserName $UserName -Password $Password

  $token.token
  $token.expires

  $jobStatus = Get-AGSJobStatus  $ServerURL -JobID $jobID -AdminToken $token.token
  $jobStatus 

  $jobStatus.status           # will be set to  "COMPLETED"
  $jobStatus.totalsteps       # totalsteps will be equal to currentstep 
  $jobStatus.currentstep
  $jobStatus.stepInfo         # detailed description of each step

} catch {
  $_
}
#>

#################################################################################
##
function Create-AGSSite([string]$ServerURL="http://localhost:6080",[string]$UserName,[string]$Password,[switch]$RunAsync,[int]$TimeOut=300)
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/createNewSite"  
  $cmdBody = "f=json&username=$UserName&password=$Password"
  if ($RunAsync) {
    $cmdBody += "&runAsync=true"
  }
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing -TimeoutSec $TimeOut
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response.jobid 
}

#################################################################################
##
function Join-AGSSite([string]$ServerURL="http://localhost:6080",[string]$AdminURL,[string]$UserName,[string]$Password,[int]$TimeOut=300)
{ 
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/joinSite"
  $cmdBody = "f=json&adminURL=$AdminURL&username=$UserName&password=$Password"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing -TimeoutSec $TimeOut
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response
}

#################################################################################
##
function Add-MachineToCluster([string]$ServerURL="http://localhost:6080",$Token,$MachineNames,$ClusterName)
{ 
  $cmdURL  = $ServerURL.TrimEnd("/") + "/clusters/$ClusterName/machines/add"
  $cmdBody = "f=json&token=$Token&machineNames=$MachineNames"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing 
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response
}

#################################################################################
##
function Unregister-Machine([string]$ServerURL="http://localhost:6080",$Token,$MachineName) 
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/machines/$MachineName/unregister"
  $cmdBody = "f=json&token=$Token"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing 
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response
}

#################################################################################
##
function List-Machines([string]$ServerURL="http://localhost:6080",$Token) 
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/machines"
  $cmdBody = "f=json&token=$Token"
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing 
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  @(,$response.machines)
}

#################################################################################
##
function Get-MachineStatus([string]$ServerURL="http://localhost:6080",$Token,$MachineName) 
{
  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/machines/$MachineName/status"
  $cmdBody = "f=json&token=$Token"      # csrfPreventToken ??
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}

  $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing 
  $response = $res.Content | ConvertFrom-Json
  Check-ResponseStatus $response 
  $response
}

### EOF


<#
try {
  $token = Get-AGSAdminToken $ServerURL  -UserName $UserName -Password $Password
} catch {
  $_
}

$token.token
$token.expires


try {
  $res = Delete-AGSSite -ServerURL $ServerURL -AdminToken $token.token
} catch {
  $_
}


try {
  $jobid = Create-AGSSite $ServerURL  -UserName $UserName -Password $Password -RunAsync
} catch {
  $_
}

$jobid

try {
  $token = Get-AGSAdminToken $ServerURL  -UserName $UserName -Password $Password
} catch {
  $_
}

$token.token
$token.expires


try {
  $jobStatus = Get-AGSJobStatus  $ServerURL -JobID $jobID -AdminToken $token.token
  $jobStatus 
} catch {
  $_
}


try {
$res = Test-AGSServer $ServerURL
} catch {
  $_
}

try {
  $res = Get-AGSSiteInfo $ServerURL
} catch {
  $_
}



$machines = List-Machines $ServerURL -Token $token.token
$machines


$status = Get-MachineStatus $ServerURL -Token $token.token  -MachineName "VM-SASHA-58.CS-SASHA-58.B4.INTERNAL.CLOUDAPP.NET"
#>


<#
Function site {
Param( $url )
$web = New-Object System.Net.WebClient
$web.UseDefaultCredentials=$true
$out = $web.DownloadString("$url")
if($out -match "access"){" $url is accessible"}else{"$url is not accessible"}
} )


Invoke-WebRequest -Uri "http://www.bing.com/" | Select-Object StatusDescription


  $uri = New-Object System.Uri($AccInfo.BlobEndPoint + $urlPath) 
  [Net.HttpWebRequest]$request = [System.Net.WebRequest]::create($uri)
  $request.Method = $requestMethod
  $request.ContentLength = 0
  $request.Headers.Add("x-ms-date", $dateInRfc1123Format)
  $request.Headers.Add("x-ms-version", $msVersion)
  $request.Headers.Add("Authorization", $authorizationHeader)
  $request.Headers.Add("Accept-Charset", "UTF-8")
  $request.Accept = "application/atom+xml,application/xml"
  return $request

  public boolean isServerAvailable() { 
    try { 
      URL url = new URL(adminURL + "/?f=json"); 
      java.net.HttpURLConnection connection = (HttpURLConnection)url.openConnection(); 
      connection.setRequestMethod("GET"); 
      connection.setDoOutput(false); 
      connection.setReadTimeout(10000); 
      connection.connect(); 
      boolean ok = (connection.getResponseCode() == 200); 
      connection.disconnect(); 
      return ok; 
    } catch(Exception ex) { 
      return false; 
    } 
  } 

#>
<#

#################################################################################
##
function Test-AGSServer([string]$ServerURL="http://localhost:6080")
{
  $cmdUrl  = $ServerURL.TrimEnd("/") + "/arcgis/admin"
  $cmdBody = "f=json"
 
 [System.Web.HttpUtility]::UrlEncode("f=json")

  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}
    
  try{  

    $res = Invoke-WebRequest -Uri $cmdUrl -Method GET -UseDefaultCredentials -UseBasicParsing -DisableKeepAlive -Verbose  | Select-Object StatusCode   

    $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -WebSession $global:sVar -UseDefaultCredentials -UseBasicParsing -Verbose #  | Select-Object StatusCode   

# -DisableKeepAlive
#    $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -SessionVariable sessionVar  -UseBasicParsing  | Select-Object StatusCode   
#$res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -WebSession $global:sVar -UseDefaultCredentials -UseBasicParsing -Verbose   | Select-Object StatusCode   
#$global:sVar = $sessionVar  
  
    if ($res -and ($res.StatusCode -eq 200)) {
      return $true
    }
  } catch { 
    ## does not matter
  }

  return $false
}

#>


#    $res = Invoke-WebRequest -Uri $cmdURL -Body $cmdBody -Headers $headers -Method Post -UseDefaultCredentials -TimeoutSec $TimeOut -UseBasicParsing  -Verbose
#    $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -WebSession $global:sVar -UseDefaultCredentials -UseBasicParsing -Verbose #  | Select-Object StatusCode   
# -DisableKeepAlive
#    $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -SessionVariable sessionVar  -UseBasicParsing  | Select-Object StatusCode   
#$res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -WebSession $global:sVar -UseDefaultCredentials -UseBasicParsing -Verbose   | Select-Object StatusCode   
#$global:sVar = $sessionVar  

<#
private string GetQLMResponse(string URL)
{
    HttpWebRequest request = WebRequest.Create(URL) as HttpWebRequest;
    request.Credentials = new NetworkCredential(Settings.Default.LicenseUser, Settings.Default.LicensePassword);
    request.KeepAlive = false;
    request.Timeout = 5000;
    request.Proxy = null;

    request.ServicePoint.ConnectionLeaseTimeout = 5000;
    request.ServicePoint.MaxIdleTime = 5000;

    // Read stream
    string responseString = String.Empty;
    try
    {
        using (WebResponse response = request.GetResponse())
        {
            using (Stream objStream = response.GetResponseStream())
            {
                using (StreamReader objReader = new StreamReader(objStream))
                {
                    responseString = objReader.ReadToEnd();
                    objReader.Close();
                }
                objStream.Flush();
                objStream.Close();
            }
            response.Close();
        }
    }
    catch (WebException ex)
    {
        throw new LicenseServerUnavailableException();
    }
    finally
    {
        request.Abort();
    }
    return responseString;
}


#################################################################################
##
function Test-WebServer([string]$ServerURL="http://localhost:6080",[int]$TimeOut=30)
{
  $cmdUrl = $ServerURL.TrimEnd("/")
  try {
    $res = Invoke-WebRequest -Uri $cmdUrl -UseBasicParsing -Method POST -TimeoutSec $TimeOut -DisableKeepAlive
    log "Status code : {0}, description : {1}" -p $res.StatusCode,$res.StatusDescription
    if ($res -and ($res.StatusCode -eq 200)) {
      return $true
    }
  } catch { 
    # $res = $_.Exception.Response
    # log "Status code : {0}, description : {1}" -p ($res.StatusCode.value__),($res.StatusDescription)
    log ($MyInvocation.MyCommand.ToString() + ': ' +  $_.Exception.Message.ToString())
  }
  return $false
}

#################################################################################
##
function Test-AGSServer-OLD([string]$ServerURL="http://localhost:6080")
{
  $cmdUrl = $ServerURL.TrimEnd("/") + "/arcgis/admin/?f=json"
  try {
    $res = Invoke-WebRequest -Uri $cmdUrl | Select-Object StatusCode,StatusDescription
    if ($res) {
      log "Status code : {0}, description : {1}" -p $res.StatusCode,$res.StatusDescription
      if ($res.StatusCode -eq 200) {
        return $true
      }
    }
  } catch { 
    # $res = $_.Exception.Response
    # log "Status code : {0}, description : {1}" -p ($res.StatusCode.value__),($res.StatusDescription)
    log "$($MyInvocation.MyCommand) : $($_.Exception.Message). Try again." -fc Gray
  }
  return $false
}

#################################################################################
##
function Get-WebSessionVariable([string]$ServerURL="http://localhost:6080")
{
  $cmdUrl  = $ServerURL.TrimEnd("/") + "/arcgis/admin"
  $cmdBody = "f=json"

  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}
  try{  
    $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method POST -Headers $headers -SessionVariable sessionVar -UseDefaultCredentials -UseBasicParsing  # -Verbose
    if ($res -and ($res.StatusCode -eq 200)) {
      return $sessionVar
    }
  } catch { 
    ## do nothing
  }

  return $null
}


#################################################################################
## TODO: use HTTPS
function Create-AGSSite-OLD([string]$ServerURL="http://localhost:6080",[string]$UserName,[string]$Password,[int]$TimeOut=300, 
                        [string]$RootDir="c:\arcgisserver")
{
  ## TODO: extend this method to set dirs,logs and connection  

  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/createNewSite"  
  $cmdBody = "f=json&username=$UserName&password=$Password"    

  [GC]::Collect()  # TODO: ?
 
  $res = @{status="succeess"}
  try {
    $res = Invoke-RestMethod -Uri $cmdURL -Body $cmdBody -Method Post -TimeoutSec $TimeOut -ContentType 'application/x-www-form-urlencoded'
  } catch {
    ## TODO: find out why timeout happens,  for now, it's safe to ignore this exception 
    #  $msg = $_.Exception.Message
    #  $_ | logr -PassThru | %{throw $_}
    log "WARNING: {0}" -p ($_.Exception.Message) -fc Gray
  }
#  Check-ResponseStatus $res
}

#################################################################################
## TODO: use HTTPS
function Create-AGSSite-Ex([string]$ServerURL="http://localhost:6080",$UserName,$Password,[int]$TimeOut=300) 
{ # TODO: re-write this method 
  $RootDir  = "c:\arcgisserver" 

# TODO:  remove debug
 $ServerURL= "http://cs-sasha-58.cloudapp.net/"
 $UserName = "siteadmin"
 $Password = "admin@vm1"
 $RootDir  = "c:\arcgisserver" 
# TODO:  remove debug

  $configStorePath = Join-Path $RootDir -ChildPath "config-store"
  $directoriesPath = Join-Path $RootDir -ChildPath "directories"

  $dirArray = @(@{ name = "arcgiscache"
                   physicalPath = Join-Path $directoriesPath -ChildPath "arcgiscache"
                   directoryType = "CACHE"
	               cleanupMode = "NONE"
	               maxFileAge = 0},
                @{ name = "arcgisjobs"
                   physicalPath = Join-Path $directoriesPath -ChildPath "arcgisjobs"
                   directoryType = "JOBS"
	               cleanupMode = "TIME_ELAPSED_SINCE_LAST_MODIFIED"
	               maxFileAge = 360},
                @{ name = "arcgisoutput"
                   physicalPath = Join-Path $directoriesPath -ChildPath "arcgisoutput"
                   directoryType = "OUTPUT"
	               cleanupMode = "TIME_ELAPSED_SINCE_LAST_MODIFIED"
	               maxFileAge = 10},
                @{ name = "arcgissystem"
                   physicalPath = Join-Path $directoriesPath -ChildPath "arcgissystem"
                   directoryType = "SYSTEM"
	               cleanupMode = "NONE"
	               maxFileAge = 0}) 
    
  $directories = @{directories = $dirArray}

  $cfgStoreCon = @{type = "FILESYSTEM" 
                   connectionString = $configStorePath}
  
  $logSettings = @{logLevel = "INFO"
                   logDir = Join-Path $rootDir -ChildPath "logs"  		
              	   maxErrorReportsCount = 10
		           maxLogFileAge = 90}

  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/createNewSite?f=json"  

  $dirs = (ConvertTo-Json $directories -Compress).toString()
  $conn = (ConvertTo-Json $cfgStoreCon -Compress).toString()
  $lset = (ConvertTo-Json $logSettings -Compress).toString()

# Tested --------------------
#  $cmdBody = "username=$UserName&password=$Password&directories=$dirs&configStoreConnection=$conn&logsSettings=$lset&runAsync=false"
#  $cmdBody = "username=$UserName&password=$Password&directories=$dirs&configStoreConnection=$conn&logsSettings=$lset"
#  $cmdBody = "username=$UserName&password=$Password&directories=$dirs&configStoreConnection=$conn"
#  $cmdBody = "username=$UserName&password=$Password&directories=$dirs&runAsync=false"
#  $cmdBody = "username=$UserName&password=$Password&directories=$dirs"
#  $cmdBody = "username=$UserName&password=$Password"

  $cmdURL  = $ServerURL.TrimEnd("/") + "/arcgis/admin/createNewSite"  
  $cmdBody = "f=json&username=$UserName&password=$Password"                       # &runAsync=true


# TODO:  remove debug
#  log -Type DEBUG -Msg "cmdUrl  : {0}" -p $cmdURL  -Simple -Debug
#  log -Type DEBUG -Msg "cmdBody : {0}" -p $cmdBody -Simple -Debug

#  $cmdBody = @{username=$UserName;password=$Password;configStoreConnection=$conn;directories=$dirs}
  
#  $cmdHdrs = @{'Content-type'='application/x-www-form-urlencoded';'Accept'='text/plain'}
#  $res = Invoke-RestMethod -Uri $cmdURL -Body $cmdBody -Headers $cmdHdrs -Method Post -TimeoutSec 20  #$TimeOut 

#[System.GC]::Collect() 
#[GC]::Collect()
log "invoke-rest  start"
#-ContentType 'application/x-www-form-urlencoded' 
  $res = $null
  try {
    $res = Invoke-RestMethod -Uri $cmdURL -Body $cmdBody -Method Post -TimeoutSec 300  # $TimeOut 
  } catch {
    ## TODO: find out why timeout happens
    #        for now, it's safe to ignore this exception 
    $msg = $_.Exception.Message
    log -Msg $msg -fc Yellow 
    $res = @{status="succeess"}
    #  $_ | logr -PassThru | %{throw $_}
  }
log "invoke-rest end"  
  $res 
  
## TODO:
#  Check-ResponseStatus $res
}

#>

<########### test 





$token = Get-AGSAdminToken $ServerURL  -UserName $UserName -Password $Password
$token.token
$token.expires

$jobid = "jb91aef2e-18d3-478f-b999-495b7d1a1c9d"
try {
  $res = Get-AGSJobStatus  $ServerURL -JobID $jobID -AdminToken $token.token
} catch {
  $_
}


try {
  $res = Create-AGSSite $ServerURL  -UserName $UserName -Password $Password -RunAsync
} catch {
  $_
}



$res = Delete-AGSSite -ServerURL $ServerURL -AdminToken $token.token
$res

$token = Get-AGSAdminToken $ServerURL  -UserName $UserName -Password $Password
$token.token
$token.expires

#$res = Test-AGSServer ($ServerURL + "/bsd" )
$res = Test-AGSServer $ServerURL

try {
#$res = Get-AGSSiteInfo ($ServerURL + "/bsd" )
$res = Get-AGSSiteInfo $ServerURL
if ( $res.StatusCode -ieq 200 ) {
  $content = $res.Content | ConvertFrom-Json
  if ($content.error) {
    $content.error.Message
  } else {
    $content
  }
} else {
  $res.StatusCode
}
 } catch { 
 # $_.Exception.Status = ProtocolError
 # $_.Exception.Message = The remote server returned an error: (404) Not Found.
 # $_.ErrorDetails.Message = HTTP Status 404 - //bsd/arcgis/rest/info
 #                           type Status report
 #                           message //bsd/arcgis/rest/info
 #                           description The requested resource is not available.
 #                           Apache Tomcat/7.0.47
   $_
  }

#################################################################################
##
function Test-AGSServer([string]$ServerURL="http://localhost:6080")
{
  $cmdUrl  = $ServerURL.TrimEnd("/") + "/arcgis/admin"
  $cmdBody = "f=json"
#(([system.Text.Encoding]::UTF8).GetBytes($cmdBody)).Count 
  $headers = @{'Content-type'='application/x-www-form-urlencoded'
               'Content-Length' = $cmdBody.Length
               'Accept' = 'text/plain'}
    
    NOT WORKING!!!!!!!!!!!!

  try{  
    $res = Invoke-WebRequest -Uri $cmdUrl -Body $cmdBody -Method GET -Headers $headers -UseDefaultCredentials -DisableKeepAlive -UseBasicParsing | Select-Object StatusCode   
    if ($res -and ($res.StatusCode -eq 200)) {
      return $true
    }
  } catch { 
    ## does not matter
  }

  return $false
}


#>