﻿if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

<#
      $sth = New-AzureStorageHelper $Storage    
      $sth.Connect()
      $sth.CreateContainer($Container)
      $sth.OpenContainer($Container)
      $sth.SetBlobContent($file,$name)   
  
#>
  
#########################################################################################
## 
function New-AzureStorageHelper([string]$StorageAccountName) 
{
  $obj = New-Object psobject
  $obj | Add-Member -MemberType NoteProperty -Name "StorageAccountName"    -Value $StorageAccountName -Force
  $obj | Add-Member -MemberType NoteProperty -Name "StorageAccountKey"     -Value ""                  -Force

  $obj | Add-Member -MemberType NoteProperty -Name "ObjCtxt" -Value $null -Force
  $obj | Add-Member -MemberType NoteProperty -Name "ObjCntr" -Value $null -Force 

  ################################################################################
  ## Init (obtain storage primary Key)
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "Init" -Force -Value {
    param()
    $this.StorageAccountKey = (Get-AzureStorageKey -StorageAccountName $this.StorageAccountName).Primary
    if (!$this.StorageAccountKey) {
      throw (fls "Bad storage key. Make sure storage account name is valid : {0}" -p $StorageAccountName)
    }
  }

  ################################################################################
  ## Connect
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "Connect" -Force -Value {
    param()
    if (!$this.StorageAccountKey) {
##      $this.Init()
      $this.StorageAccountKey = (Get-AzureStorageKey -StorageAccountName $this.StorageAccountName).Primary
    }
    Log-Message "connecting to storage account : {0}" -p $this.StorageAccountName 
    $this.ObjCtxt = New-AzureStorageContext -StorageAccountName $this.StorageAccountName -StorageAccountKey $this.StorageAccountKey -ErrorAction Stop
  }
  
  ################################################################################
  ## ListContainers
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "ListContainers" -Force -Value {
    param([string]$Prefix="*")
    if (!$this.ObjCtxt) { 
      throw (fls "Connection must be set prior to any operations on the storage account")
    }  
    $this.ObjCtxt | Get-AzureStorageContainer $Prefix | ?{$_.PublicAccess -ieq "container"}
  }

  ################################################################################
  ## Test Container's existence
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "TestContainer" -Force -Value {
    param([string]$ContainerName)
    if (!$this.ObjCtxt) { 
      throw (fls "Connection must be set prior to any operations on the storage account")
    }  
    
    $arr = $this.ListContainers($ContainerName)
    if ($arr -and $arr.Count -gt 0) {
      return $true
    }
    return $false
  }
  
  ################################################################################
  ## CreateContainer
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "CreateContainer" -Force -Value {
    param([string]$ContainerName)
    if (!$this.ObjCtxt) { 
      throw (fls "Connection must be set prior to any operations on the storage account")
    }  
    Log-Message "creating container : {0}" -p $ContainerName
    $this.ObjCtxt | New-AzureStorageContainer -Name $ContainerName -Permission Container -ErrorAction Stop -Verbose | Out-Null
  }

  ################################################################################
  ## RemoveContainer
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "RemoveContainer" -Force -Value {
    param([string]$ContainerName,[switch]$Force)
    if (!$this.ObjCtxt) { 
      throw (fls "Connection must be set prior to any operations on the storage account")
    }  
    Log-Message "removing container : {0}" -p $ContainerName
    $this.ObjCtxt | Remove-AzureStorageContainer -Name $ContainerName -Force:$Force -Verbose
  }

  ################################################################################
  ## OpenContainer
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "OpenContainer" -Force -Value {
    param([string]$ContainerName)
    if (!$this.ObjCtxt) { 
      throw (fls "Connection must be set prior to any operations on the storage account")
    }  
    Log-Message "opening container : {0}" -p $ContainerName
    $this.ObjCntr = $this.ObjCtxt | Get-AzureStorageContainer -Name $ContainerName
  }

  ################################################################################
  ## ListBlobs
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "ListBlobs" -Force -Value {
    param([string]$BlobPrefix="*")
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    $prop1 = @{Name="ETag";         Expression={$_.ICloudBlob.Properties.ETag.Trim("`"")}}
    $prop2 = @{Name="MD5";          Expression={$_.ICloudBlob.Properties.ContentMD5}}
    $prop3 = @{Name="ICloudBlob";   Expression={$_.ICloudBlob}}     # TODO: ???
    $arr = @($this.ObjCntr | Get-AzureStorageBlob -Blob $BlobPrefix -ErrorAction SilentlyContinue | select-object Name,Length,LastModified,$prop1,$prop2,$prop3)  
    return ,$arr
  }
  
  ################################################################################
  ## GetBlob
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "GetBlob" -Force -Value {
    param([string]$BlobName)
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    $this.ObjCntr | Get-AzureStorageBlob -Blob $BlobName -ErrorAction SilentlyContinue
  }
   
  ################################################################################
  ## TestBlob
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "TestBlob" -Force -Value {
    param([string]$BlobName,[string]$MD5,[long]$Length,$LastModified)
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    $blob = $this.GetBlob($BlobName)
    if (!$blob) {
      return $false
    }
    # check blob's md5 (optional)      
    if ($MD5 -and ($blob.ICloudBlob.Properties.ContentMD5 -ine $MD5)) {
      return $false
    }
    # check blob's size (optional)      
    if ($Length -and ($blob.Length -ine $Length)) {
      return $false
    }
    # check last modified time stamp (optional)
    if ($LastModified -and ($blob.LastModified.CompareTo($LastModified) -ne 0)) {
      return $false
    }
    # passed all checks
    return $true
  }

  ################################################################################
  ## GetBlobContent
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "GetBlobContent" -Force -Value {
    param([string]$BlobName,[string]$File)
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    Log-Message "downloading blob {0} to file {1}" -p $BlobName,$File -fc Yellow
    $this.ObjCntr | Get-AzureStorageBlobContent -Blob $BlobName -Destination $File -Force | Out-Null 
  }

  ################################################################################
  ## SetBlobContent
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "SetBlobContent" -Force -Value {
    param([string]$File,[string]$BlobName)
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    Log-Message "uploading file {0}" -p $File -fc Yellow
    $this.ObjCntr | Set-AzureStorageBlobContent -File $File -Blob $BlobName -BlobType Block -Force | Out-Null 
  }

  ################################################################################
  ## RemoveBlob
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "RemoveBlob" -Force -Value {
    param([string]$BlobName,[switch]$Force)
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    if ($Force) {
      try {
        $blob = $this.ObjCntr | Get-AzureStorageBlob -Blob $BlobName -ErrorAction SilentlyContinue
        if ($blob) {
          $blob.ICloudBlob.BreakLease(0)
        }
      } catch {}
    }
    $this.ObjCntr | Remove-AzureStorageBlob $BlobName -Force:$Force -Verbose  *>&1 | logr
  }
  
  ################################################################################
  ## RemoveBlobs
  #
  $obj | Add-Member -MemberType ScriptMethod -Name "RemoveBlobs" -Force -Value {
    param([string]$BlobPrefix,[switch]$Force)
    if (!$this.ObjCntr) { 
      throw (fls "Container must be opened before you can access blobs inside it")    
    }  
    $blobs = $this.ListBlobs($BlobPrefix)
    if ($blobs) {
      Log-Message "Removing blobs '{0}' from container '{1}'" -p $BlobPrefix,$this.ObjCntr.Name -fc Yellow
      foreach ($blob in $blobs) {
        $this.RemoveBlob($blob.Name,$Force)
      }
    }
  }

  return $obj
}

### EOF