﻿if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

#########################################################################################
## TODO: cleanup
##  
function Create-AGSVM()
{
  param(
    ##
    ## Optional, in case if you want to re-connect to Azure 
    ##
    [string]$SubscriptionName,
    [string]$PublishSettingsFile,

    ##
    ## Required to know where to upload CustomScriptExtension resources 
    ##
    [Parameter(Mandatory=$true, HelpMessage='Storage Account to use for the image generated')]
    [ValidateNotNullOrEmpty()]
    [string]$StorageAccountName,

    ##
    ## Name and location for a cloud service the new VM will be associated with 
    ##    - If service name is not provided, an random name will be generated
    ##    - If service does not exist, it will be created automatically  
    ##
    [string]$CloudServiceName = ("cs-{0}" -f ((([System.Guid]::NewGuid()).Guid).Replace('-',''))),     
    [ValidateSet("East Asia","Southeast Asia","North Europe","West Europe","East US","North Central US","South Central US","West US","Japan East","Japan West","Brazil South")]
    [string]$CloudServiceLocation = "East US",

    ##
    ## Azure Image to create VM from, can be a stock MS image or AGS Image  
    ##

    [Parameter(HelpMessage='Name of the Image to create VM from. Either Name or Label must be provided in the input.')]
    [string]$ImageName,          
    [Parameter(HelpMessage='Label of the Image to create VM from. Either Name or Label must be provided in the input.')]
    [string]$ImageLabel = "Windows Server 2012 R2 Datacenter, March 2014",

    ##
    ## VM properties 
    ##
    [Parameter(Mandatory=$false, HelpMessage='The new Virtual Machine name. Lower case, 15 characters length limit.')]
    [string]$VMachineName     = ("vm-" + (([System.Guid]::NewGuid()).Guid).Replace('-','').Substring(0,12).ToLower()),   
    [string]$VMachineTimeZone = "Pacific Standard Time",
    [string]$VMachineSize     = "Medium",

    [Parameter(HelpMessage='Provide a known Admin credentials if you plan to access the VM later.')]
    [string]$VMAdminName       = ((([System.Guid]::NewGuid()).Guid).Replace('-','').Substring(0,20)),
    [string]$VMAdminPassword   = ((([System.Guid]::NewGuid()).Guid).Replace('-','')),

    ##
    ## The following parameters define operations that will be executed on newly created VM  
    ##

    # OPERATION: InstallAGS
    [Parameter(Mandatory=$false, HelpMessage='Absolute Path to the location of your ArcGIS_Server setup folder')]
    [string]$AGSSetupPath,
    
    # OPERATION: AuthorizeAGS
    [Parameter(Mandatory=$false, HelpMessage='Absolute Path to your ArcGIS Server License file (license.prvc)')]
    [string]$AGSLicensePath,

    # OPERATION: InstallSSLCert
    [Parameter(Mandatory=$false, HelpMessage='Absolute Path to your SSL certificate file')]
    [string]$SSLCertificatePath,  

    # OPERATION: InstallSQLClient 
    [Parameter(Mandatory=$false, HelpMessage='Absolute Path to your SQL Client MSI installer')]
    [string]$SQLClientPath,  

    # OPERATION: CreateSite
    [Parameter(Mandatory=$false, HelpMessage='Create site during VM provisioning. Admin credentials are required for this operation.')]
    [switch]$CreateSite,
    [string]$SiteAdminName,        # both admin name and password are required if CreateSite switch is on
    [string]$SiteAdminPassword,
      
    # OPERATION: ConfigureFirewall
     [string]$Firewall,   

    # OPERATION: RunSysprep
    [switch]$Sysprep, 

    # OPERATION: AddPublicEndpoints
    [switch]$AGSEndpoints, 

    ##
    ## Controling parameters  
    ##
    [switch]$Wait,       # blocks the call untill VM is fully configured (all operations completed)
    [switch]$DontClean   # do not attemps to remove all temporary created resources  
  )

  $agsRes = $null
  try
  {
    Connect-To-Azure $SubscriptionName $PublishSettingsFile $StorageAccountName
  
    $vm = Get-AzureVM -ServiceName $CloudServiceName -Name $VMachineName -ErrorAction SilentlyContinue  *>&1 | logr
    if ($vm) 
    {
      log "virtual machine {0} already exists in service {1}" -p $VMachineName,$CloudServiceName
      if ($AGSEndpoints ) {
        Add-AGSEndpoints -VM $vm 
      }
      return  #  ? $vm
    }

    $image = Get-AzureImage -Name $ImageName -Label $ImageLabel
    log "Azure Image : {0}" -p $image.ImageName

    # if service does not exist - create it
    Create-NewAzureService  $CloudServiceName $CloudServiceLocation

    $agsRes = Create-AGSCustomScriptExtensionResources -StorageAccountName $StorageAccountName `
                                                       -AGSSetupPath $AgsSetupPath `
                                                       -AGSLicensePath $AGSLicensePath `
                                                       -CreateSite:$CreateSite -SiteAdminName $SiteAdminName -SiteAdminPassword $SiteAdminPassword `
                                                       -Firewall $Firewall `
                                                       -SQLClient $SQLClientPath `
                                                       -SSLCertificatePath $SSLCertificatePath `
                                                       -Sysprep:$Sysprep

    log "creating VM configuration" -fc Yellow

    $vmCfg = New-AzureVMConfig -Name $VMachineName -InstanceSize $VMachineSize -ImageName $image.ImageName -Verbose  |
             Add-AzureProvisioningConfig -Windows -TimeZone $VMachineTimeZone -AdminUsername $VMAdminName -Password $VMAdminPassword -Verbose |
             Set-AGSCustomScriptExtension -res $agsRes


    log ($vmCfg | fl | Out-String) -Simple
    log "VM configuration succesfully created" -fc Green

    log "starting VM instance : {0}" -p $VMachineName -fc Yellow

    $numberOfAttempts = 10
    $sleepIntervalSec = 2     
    while ($numberOfAttempts -gt 0) 
    {
      try {
        $vm = $vmCfg | New-AzureVM -ServiceName $CloudServiceName -WaitForBoot -ErrorAction Stop -Verbose  *>&1 | logr
        break
      } catch {
        $_ | logr 
        log "failed, trying again in {0} seconds. Attempt counter : {1}" -p $sleepIntervalSec,$numberOfAttempts -fc DarkYellow
        $numberOfAttempts--
        Sleep -Seconds $sleepIntervalSec
      }   
    }

    if ($numberOfAttempts -gt 0) {
      log "VM instance is started : {0}" -p $VMachineName -fc Green
    } else {
      log "VM instance startup failed : {0}" -p $VMachineName -fc red
      throw $Error[0]  # last thrown eception
    }

    # VM is running, wait for all the AGS operations to finish
    if ($agsRes -and $Wait) 
    {
      # if syspreping, wait for machine to stop
      if ($Sysprep) {
        Wait-VMUntilStopped -CSName $CloudServiceName -VMName $VMachineName
      } 
    }

    # set endpoints 
    if ($AGSEndpoints -and !$Sysprep) {
      Add-AGSEndpoints -CSName $CloudServiceName -VMName $VMachineName
    }
  }
  catch
  {
    $err = log "Create-AGSVM : FAILED" -fc Red -PassThru
    $_ | logr 
    throw $err
  }
  finally ### CLEANING ####
  {
    if ($agsRes -and !$DontClean) {
      Remove-AGSCustomScriptExtensionResources $agsRes
    }
  }
}

###############################################################################
<# 
.Synopsis
   Find Azure Image by Name or Label.
#>
function Get-AzureImage
{   
  Param ([Alias("ImageName","n")][string]$Name,
         [Alias("ImageLabel","l")][string]$Label
        )

  $img = $null
  if ($Name) {
    $msg = fls "find Azure Image by Name : {0}"  -p $Name

    $img = Get-AzureVMImage -ImageName $Name -ErrorAction SilentlyContinue
    if (!$img) {
      throw (fls "Image {0} not found" -p $Name)
    }
  } elseif ($Label) {
    $img = Get-AzureVMImage -ErrorAction SilentlyContinue | Where-Object {($_.Label -ieq $Label)}
    if (!$img) {
      throw (fls "Image {0} not found" -p $Label)
    }
  } else {   # neither name nor label were provided, try to find the latest image of "Windows Server 2012 R2 Datacenter"
    $str = "Windows Server 2012 R2 Datacenter"
    $img = Get-AzureVMImage -ErrorAction SilentlyContinue | Where {$_.Label.Contains($str)} | Sort -Property PublishedDate -Descending | select -First 1
    if (!$img) {
      throw (fls "Image with label containing '{0}' not found" -p $str)
    }
  }

  return $img
}

#################################################################################
## 
function Test-AzureImage([string]$Name,[string]$Label)
{   
  try {
    $img = Get-AzureImage -Name $Name -Label $Label
    if ($img) {
      return $true
    }
  } catch {
  }
  return $false
}

#########################################################################################
##
function Add-AGSEndpoints
{
  Param ( [Alias("VMObject")]$VM,
          [Alias("ServiceName","csn")][string]$CSName,
          [Alias("MachineName","vmn")][string]$VMName
        )

  if (!$VM -and ($CSName -and $VMName)) {
    $VM = Get-AzureVM -ServiceName $CSName -Name $VMName 
  }
  if (!$VM) {
    #log (fls "Invalid VM")   ## TODO: Should we hard fail or try to repeat the operation ?
    throw (fls "Invalid VM")
  }

  ## standard AGS endpoints
  ##
  $all = @(@{Name= 'agsHTTP';  Protocol= 'tcp'; PublicPort=  80; LocalPort= 6080; Verbose= $true}
           @{Name= 'agsHTTPS'; Protocol= 'tcp'; PublicPort= 443; LocalPort= 6443; Verbose= $true})


  ## add only those that has not been added before
  ##
  $atLeastOneIsSet = $false
  foreach ($item in $all) 
  {
    if (!(Get-AzureEndpoint -Name $item.Name -VM $VM)) 
    {
      log "Adding AGS public endpoint {0} to machine : {1}" -p $item.Name, $VM.Name
      Add-AzureEndpoint -VM $VM  @Item
      $atLeastOneIsSet = $true
    }
  }
  
  ## any changes = update VM
  ##
  if ($atLeastOneIsSet) {
    $VM | Update-AzureVM -Verbose
    log "Endpoints added" -fc Green
  }
}

#########################################################################################
##
function Add-AGSEndpoint
{
  [CmdletBinding()]
  Param ( [Parameter(ValueFromPipeline=$true)][Alias("VMObject")]$VM,
          [Alias("ServiceName","csn")][string]$CSName,
          [Alias("MachineName","vmn")][string]$VMName,
          [string]$Name,[string]$Protocol,[int]$PublicPort,[int]$LocalPort 
        )
  Begin
  {
    if (!$VM -and ($CSName -and $VMName)) {
      $VM = Get-AzureVM -ServiceName $CSName -Name $VMName -Verbose
    }
  }
  Process
  {
    if ($VM) 
    {
      $endp = Get-AzureEndpoint -Name $Name -VM $VM 
      if($endp -eq $null) {
        $VM | Add-AzureEndpoint -Name $Name -Protocol $Protocol -PublicPort $PublicPort -LocalPort $LocalPort -Verbose 
      } else {
        $VM 
      } 
    }
  }
  End
  {
  }
}

#########################################################################################
##
function Get-CurrentFunctionName([string]$Text)
{
  # 0 - this func, 1 - parent, 2 - grand-parent etc  
  # since when we call this function "current" is actually "parrent", scope == 1
  $mi = Get-Variable MyInvocation -Scope 1  
  if ($Text) {
    $mi.Value.MyCommand.Name + " " + $Text
  } else {
    $mi.Value.MyCommand.Name
  }
}

#########################################################################################
##
function Get-ParentFunctionName([string]$Text)
{
  $mi = Get-Variable MyInvocation -Scope 2  
  if ($Text) {
    $mi.Value.MyCommand.Name + " " + $Text
  } else {
    $mi.Value.MyCommand.Name
  }
}

#################################################################################
## 
function Test-PSVersion([System.Version]$MinAcceptable)
{
#    Log-Message "Checking current {0} version" -p "PowerShell"
    Log-Message "Current {0} version : {1}" -p "PowerShell", $PSVersionTable.PSVersion
  if ($PSVersionTable.PSVersion.CompareTo($MinAcceptable) -lt 0) {
    throw (fls "Wrong {0} version, this script requires version {1} or above. Abort." -p "PowerShell",$MinAcceptable)
  }
}

#################################################################################
## 
function Test-AzureVersion([System.Version]$MinAcceptable)
{
#  Log-Message "Checking current {0} version" -p "Azure"
  $azModDef = "C:\Program Files (x86)\Microsoft SDKs\Windows Azure\PowerShell\ServiceManagement\Azure\Azure.psd1"
  $azMod = Import-Module $azModDef -ErrorAction Stop -PassThru
  Log-Message "Current {0} version : {1}" -p "Azure",$azMod.Version
  if ($azMod.Version.CompareTo($MinAcceptable) -lt 0) {
    throw (fls "Wrong {0} version, this script requires version {1} or above. Abort." -p "Azure",$MinAcceptable)
  }
}

#################################################################################
## 
function Test-CloudServiceLocation($Location)
{
#  log "Checking cloud service location : {0}" -p $Location
  $azLoc = Get-AzureLocation | Where {$_.Name -ieq $Location}
  if (!$azLoc) {
    throw (fls "Location {0} is not in the list of locations available for current subscription. Try another location." -p $Location)
  }
# log "cloud service location details : `n{0}" -p ($azLoc | fl | Out-String)
# log "Succeeded" -fc Green
}

#################################################################################
## 
function Wait-VMUntilStopped([string]$CSName,[string]$VMName,[int]$SleepSec=30)
{
  log "Waiting till machine {0} is stopped, checking every {1}sec..." -p $VMName,$SleepSec -fc Yellow
  while ( 1 ) 
  {
    $vm = $null
    $vm = Get-AzureVM -ServiceName $CSName -Name $VMName -ErrorAction SilentlyContinue
#    if ($vm -and $vm.InstanceStatus -and ($vm.InstanceStatus -ieq 'StoppedVM')) { 
    if ($vm -and ($vm.PowerState -eq "Stopped")) { 
      break
    }
    Start-Sleep -Seconds $SleepSec 
  }  
  log "VM is stopped" -fc Green
}

#################################################################################
## 
function Create-NewAzureService([string]$CloudServiceName,[string]$CloudServiceLocation,[int]$MaxAttempts=20,[int]$DelayInSeconds=10)
{
  # if service does not exist - create it
  while ($MaxAttempts -gt 0 -and !(Test-AzureName -Service -Name $CloudServiceName)) 
  {
    try {
      log "creating service {0}, location {1}" -p $CloudServiceName,$CloudServiceLocation -fc Yellow
  
      New-AzureService -ServiceName $CloudServiceName -Location $CloudServiceLocation -Verbose   *>&1 | logr
      
      log "cloud service created : {0}" -p $CloudServiceName -fc Green
      return
    } catch {
      $_ | logr
      log "New-AzureService failed, trying again in {0} seconds. Attempts counter : {1}" -p $DelayInSeconds,$MaxAttempts -fc DarkYellow
      $MaxAttempts--
      Sleep -Seconds $DelayInSeconds
    }
  }
}

#################################################################################
## Connection is made for a new set of input param
## Repeated call ignored unless Forced
function Connect-To-Azure([string]$SubscriptionName,[string]$PublishSettingsFile,[string]$StorageAccountName,[switch]$Force)
{
  # silentely continue in case if some params are undefined, connection is probably already set
  if (!$SubscriptionName -or !$PublishSettingsFile -or !$StorageAccountName) {
    return
  }

  $con = $SubscriptionName,$PublishSettingsFile,$StorageAccountName -join ";"
  
  if ($Force.IsPresent -or ($global:esriCurrentAzureConnection -ine  $con)) 
  {
    $global:esriCurrentAzureConnection = $null
    log "Connecting to subscription {0}" -p $SubscriptionName

    if (!(Test-Path $PublishSettingsFile)) {
      throw (fls "Publish settings file not found : {0}" -p $PublishSettingsFile)
    }  
                                                                                                                                      
    Import-AzurePublishSettingsFile $PublishSettingsFile -ErrorAction Stop -Verbose   *>&1 | logr
    Set-AzureSubscription -SubscriptionName $SubscriptionName -CurrentStorageAccountName $StorageAccountName -ErrorAction Stop -Verbose    *>&1 | logr
    Select-AzureSubscription -SubscriptionName $SubscriptionName -Current -ErrorAction Stop -Verbose   *>&1 | logr

    # test storage account
    if (!(Test-AzureName -Storage  -Name $StorageAccountName)) {
      throw (fls "Storage account {0} not found" -p $StorageAccountName) 
    }

    $global:esriCurrentAzureConnection = $con
  }
}

#################################################################################
##
function Test-AGSSetupPath($Path)
{
## TODO: make sure that $Path points to valid AGS setup, and not to some arbitrary dir or exe
  if (!(Test-Path $Path)) {                                                                                                      
    throw (fls "AGS Setup path not found : {0}" -p $Path)
  }
}

#################################################################################
## 
function Test-AGSLicenseFile($Path)
{
  ## TODO: make sure that $File points to a valid AGS license, and not to some arbitrary file
  if (!(Test-Path $Path)) {                                                                                                      
    throw (fls "AGS license file not found : {0}" -p $Path)
  }
}

### EOF