﻿if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
  . ".\agsLogTools.ps1"
}
Load-L10NMessageTable $MyInvocation.MyCommand.Name

#########################################################################################
##
function Create-AGSCustomScriptExtensionResources
{
  Param(
    [string]$ReferenceName="AGSExtension",
    [string]$StorageAccountName,
    [string]$ContainerNamePrefix="wsc-tmp-",
    
    [string]$AGSSetupPath,         # path to ArcGIS_Server setup folder
    [string]$AGSLicensePath,       # full path
    [string]$SSLCertificatePath,   # full path 
    [string]$SQLClient,
    [switch]$CreateSite,
    [string]$SiteAdminName,        # both admin name and password are required if CreateSite switch is on
    [string]$SiteAdminPassword,
    [switch]$Sysprep,
    [switch]$SysprepAsynch,
    [switch]$CleanSite,
    [string]$Firewall,
    [ValidateSet("Never","Always","CheckMD5")]$Overwrite="Always"   # overwrite existing blobs    
    )

  Log-PSBoundParams

  ## AGS custom extension handler 
  ##
  $runScript  = Join-Path $pwd -ChildPath "Configure-AGSMachine.ps1"     
  if (!(Test-Path $runScript)) {
    throw (fls "File not found : {0}" -p $runScript)
  }
   
  $depScripts = @((Join-Path $pwd -ChildPath "agsLogTools.ps1"),         # - logger 
                  (Join-Path $pwd -ChildPath "agsAdminRESTTools.ps1"))   # - site admin tools TODO: move to 'CreateSite' section

  $depScripts | %{ if (!(Test-Path $_)) { throw (fls "File not found : {0}" -p $_) }}

  ## a random string to use in names of temporary objects (files, container name etc)
  $rnd = Create-RandomString
  log "random string : {0}" -p $rnd
  $tmpDir = Join-Path $pwd -ChildPath "tmp-$rnd"

  $extParams = ""              # custom ext argument string
  $extResourceFiles = @()      # names of files to be used for custom extension

  ### 
  ## AGS setup
  #
  if ($AGSSetupPath) {
    if (!(Test-Path $AGSSetupPath)) {
      throw (fls "AGS setup path not found : {0}" -p $AGSSetupPath)
    }
    ## setup can be a file (exe) or a folder, if it's a folder, copy it into a tmp folder and compress
    ## it into a zip file (NOTE: the folder will be removed after extension's execution is finished)
    $agsFile = Create-AGSSetupTransferFile $AgsSetupPath -TempPath $tmpDir
    $extResourceFiles += @($agsFile)
    $extParams += " -AGSSetup $(Split-Path $agsFile -Leaf)"  
    ##
    ## if AGS Setup is EXE, we will need 7z to unpack it
    ##  
    if ($agsFile.endsWith(".exe")) 
    {
      $7zDir = Join-Path $pwd -ChildPath "7Zip"
      foreach ($fname in "7z.exe","7z.dll")       # 2 extra files
      {
        $fpath = Join-Path $7zDir -ChildPath $fname
        if (!(Test-Path $fpath)) { 
          throw (fls "File not found : {0}" -p $fpath) 
        }
        $extResourceFiles += @($fpath)
      } 
    }
  }

  ###                
  ## AGS authorization
  #
  if ($AGSLicensePath) {
    if (!(Test-Path $AGSLicensePath)) {
      throw (fls "AGS Licence file not found : {0}" -p $AGSLicensePath)
    }
    $extResourceFiles += @($AGSLicensePath)
    $extParams += " -AGSLicense $(Split-Path $AGSLicensePath -Leaf)"
  }
  
  ###
  ## SSL cert update
  #
  if ($SSLCertificatePath) {
    if (!(Test-Path $SSLCertificatePath)) {
      throw (fls "SSL certificate file not found : {0}" -p $SSLCertificatePath)
    }
    $extResourceFiles += @($SSLCertificatePath)
    $extParams += " -SSLCert $(Split-Path $SSLCertificatePath -Leaf)"
  }

  
  ###
  ## SQL Client Library
  #
  if ($SQLClient) {
    if (!(Test-Path $SQLClient)) {
      throw (fls "SQL client library file not found : {0}" -p $SQLClient)
    }
    $extResourceFiles += @($SQLClient)
    $extParams += " -SQLClient $(Split-Path $SQLClient -Leaf)"
  }


  ###
  ## Create Site
  # 
  if ($CreateSite) {
    if (!$SiteAdminName -or !$SiteAdminPassword) {
      throw (fls "Wrong input parameters. Both SiteAdminName and SiteAdminPassword are required to create AGS site.")
    }
    $extParams += " -CreateSite -AdminName $SiteAdminName -AdminPassword $SiteAdminPassword"
  }

  ###
  ## Firewall settings
  #
  if ($Firewall) {
    $extParams += " -Firewall $Firewall"
  }   

  ###
  ## Sysprep
  #
  if ($Sysprep) {
    $extParams += " -Sysprep"
  } elseif ($SysprepAsynch)  {
    $extParams += " -SysprepAsynch"
  } 

  ###
  ## CleanSite
  #
  if ($CleanSite) {
    $extParams += " -CleanSite"
  }   

  # if no parameters were found in the input - quit
  if ($extParams.Length -eq 0) {
    return $null
  } 

  ###
  ## Success Signature (a keyword that will be printed to output to signal that ext finished successfully)
  #
  $SigOK = $rnd.Substring(0,8)              #   "555-555"   # Create-RandomString
  $extParams += " -SignalOK $SigOK"

  log "Processing AGS extension : {0}" -p $extParams
  $res = Create-AzureVMCustomScriptExtensionResources -RefName $ReferenceName `
                                                      -Storage $StorageAccountName `
                                                      -Container ($ContainerNamePrefix + $rnd) `
                                                      -ScriptFile $runScript  `
                                                      -OtherFiles ($depScripts + $extResourceFiles) `
                                                      -Params "-LogFile c:\AGSLogs\ags-extension.log $extParams"  `
                                                      -Overwrite $Overwrite
  # AGS resource table
  $agsRes = @{CustomScriptExtension = $res
              TmpDir   = $tmpDir
              SignalOK = $SigOK
              TmpFile  = (Join-Path $pwd -ChildPath "agsRes-$rnd.txt")}

  # save resource table in a file in current folder
  $agsRes | ConvertTo-Json -Depth 10 | Out-File -FilePath $agsRes.TmpFile
  return $agsRes
}

#########################################################################################
##
##  Creates and uploads resources used by Set-AzureVMCustomScriptExtension 
##  returns a table of resources to clean up later
##
function Create-AzureVMCustomScriptExtensionResources
{
  [CmdletBinding()]
  Param ([Alias("RefName","Name","n")][string]$ReferenceName,
         [Alias("Storage","s")][string]$StorageAccountName,
         [Alias("Container","c")][string]$ContainerName,
         [Alias("Run","r")][string]$ScriptFile,                # full path to local file
         [Alias("FileName","f")][string[]]$OtherFiles,         # all full pathes    
         [Alias("Argument","p")][string]$Params,
         [ValidateSet("Never","Always","CheckMD5")]$Overwrite="Always")   # overwrite existing blobs    

  log "Creating Azure VM Custom Script Extension Resources"
  log "Reference name : {0}" -p $ReferenceName

  #######
  ## NOTE: same as Set-AzureVMCustomScriptExtension input params, so we can later splat the table 
  #######
  $res =  @{ ReferenceName      = $ReferenceName 
             StorageAccountName = $StorageAccountName
             ContainerName      = $ContainerName 
             Run                = Split-Path $ScriptFile -Leaf
             FileName           = [string[]]$null
             Argument           = $Params}
    
  $sth = New-AzureStorageHelper $StorageAccountName
  $sth.Connect()
  if (!$sth.TestContainer($ContainerName)) {  # create new if does not exist 
    $sth.CreateContainer($ContainerName)
  }
  $sth.OpenContainer($ContainerName)

  $allFiles = @($ScriptFile) + $OtherFiles
  $allNames = @()

  foreach ($file in $allFiles) {   
    if (!(Test-Path $file)) {
      throw (fls "File not found {0}" -p $file)
    }
    $name = Split-Path $file -Leaf
    #
    # check overwrite mode, upload files
    #
    if ($Overwrite -ieq "never" -and $sth.TestBlob($name)) {  
      log "WARNING: Overwrite=never and blob {0} already exists. Skip upload." -p $name -fc Yellow
    } elseif ($Overwrite -ieq "CheckMD5" -and $sth.TestBlob($name,(Get-MD5Hash $file))) {
      log "WARNING: Overwrite=CheckMD5 and blob {0} hash is the same as of file {1}. Skip upload." -p $name,$file
    } else {
      $sth.SetBlobContent($file,$name)   
    }
    # add to the list no matter if upload operation was done or the file is already there
    $allNames += @($name) 
  }

  $res.FileName = [string[]]$allNames  # TRICK! otherwise the table does not splat correctly
  return $res  
}

#########################################################################################
##
function Remove-AzureVMCustomScriptExtensionResources($Resources,[switch]$KeepContainer)
{
  log "cleaning cloud storage resources : {0}" -p $Resources.ReferenceName
  $sth = New-AzureStorageHelper $Resources.StorageAccountName    
  $sth.Connect()
  if ($KeepContainer) {
    # removing blobs only 
    $sth.OpenContainer($Resources.ContainerName)
    foreach ($name in $Resources.FileName) {
      $sth.RemoveBlob($name,$true)  # Force      
    }
  } else {
    # removing whole container 
    $sth.RemoveContainer($Resources.ContainerName,$true)  # Force
  }
  $sth = $null
}

#########################################################################################
##
function Remove-AGSCustomScriptExtensionResources($AGSRes)
{
  $res = $AGSRes.CustomScriptExtension
  log "Cleaning  : {0}" -p $res.ReferenceName
  Remove-AzureVMCustomScriptExtensionResources $res 

  # remove temp dir
  if ($AGSRes.TmpDir -and (Test-Path $AGSRes.TmpDir -IsValid)) {
    if (Test-Path $AGSRes.TmpDir) {
      log "cleaing local resources : {0}" -p $AGSRes.TmpDir
      Remove-Item -Path $AGSRes.TmpDir -Recurse -Force -ErrorAction SilentlyContinue
    } 
  }

  # remove temp file
  if ($AGSRes.TmpFile -and (Test-Path $AGSRes.TmpFile -IsValid)) {
    if (Test-Path $AGSRes.TmpFile) {
      log "cleaing local resources : {0}" -p $AGSRes.TmpFile
      Remove-Item -Path $AGSRes.TmpFile -Force -ErrorAction SilentlyContinue
    } 
  }
}

#########################################################################################
##
function Set-AGSCustomScriptExtension
{
  [CmdletBinding()]
  Param ( [Parameter(ValueFromPipeline=$true)][Alias("VMObject")]$VM,
          [Alias("ServiceName", "csn","s")][string]$CSName,
          [Alias("MachineName", "vmn","m")][string]$VMName,
          [Alias("AGSResources","res","r")][hashtable]$AGSRes  # ext res table, created by Create-AGSCustomScriptExtensionResources()
         )
  Begin
  {
    if (!$VM -and ($CSName -and $VMName)) {
      $VM = Get-AzureVM -ServiceName $CSName -Name $VMName -Verbose # -ErrorAction Stop
    }
  }
  Process
  {
    if ($VM) {
      if ($AGSRes) {
        $res = $AGSRes.CustomScriptExtension
        $VM | Set-AzureVMCustomScriptExtension @res 
      } else {
        $VM
      }
    }
  }
}

#################################################################################
# 
# if the path points to a folder with uncompressed setup - compress it back into a single zip file
# if the path points to an EXE - assume this is the original, compressed installer - use it as is
#
function Create-AGSSetupTransferFile([string]$AgsSetupPath,[string]$TempPath)
{
  if (Test-Path $AgsSetupPath -PathType Leaf) 
  {
    if ($AgsSetupPath.EndsWith(".exe")) {         ## original AGS installer
      return $AgsSetupPath
    } elseif ($AgsSetupPath.EndsWith(".zip")) {   ## assume that ZIP file contains AGS  ## TODO: do we need this option?
      return $AgsSetupPath
    } else {
      throw (fls "ags setup path points to a file which does not appear to be a correct AGS installer : {0}" -p $AgsSetupPath)
    } 
  } 
  
  ## the path points to a folder
  #
  $testFile = "Setup.exe"
  $fileInfo = Get-ChildItem $AgsSetupPath -Recurse -Include Setup.exe | select -First 1
  if (!$fileInfo) {
    throw (fls "Folder {0} does not appear to be a valid AGS Setup Folder, a crucial component is missing : {1}" -p $AgsSetupPath, $testFile)
  }

  if (!(Test-Path $TempPath)) {
    Log-Message "creating temp folder : {0}" -p $TempPath
    New-Item -Path $TempPath -Type directory -Force  | Out-Null
      
    Log-Message "copy all the files from AGS setup dir to the temporary folder (takes some time)" -fc Yellow
    Copy-Item $AgsSetupPath -Destination $TempPath -Force -Recurse 
  }

  $setupDirCopy = Join-path $TempPath -ChildPath (Split-Path $AgsSetupPath -Leaf)
  Log-Message "setup dir : {0}" -p $setupDirCopy

  ## compress AGS setp into a zip file 
  ##  
  $zipFile = $setupDirCopy + ".zip" 
  if (!(Test-Path $zipFile)) {
    Log-Message "creating zip file : {0}" -p $zipFile -fc Yellow
    [Reflection.Assembly]::LoadWithPartialName("System.IO.Compression.FileSystem") | Out-Null
    [System.IO.Compression.ZipFile]::CreateFromDirectory($setupDirCopy,$zipFile,([System.IO.Compression.CompressionLevel]::Optimal),$true) | Out-Null
    Log-Message "zip file created" -fc Green
  }
  return $zipFile
}  

#########################################################################################
##
function Get-MD5Hash($path)
{
  $file = Resolve-Path $path
  $md5CSP = new-object -TypeName System.Security.Cryptography.MD5CryptoServiceProvider
  $fptr = [System.IO.File]::Open($file,[System.IO.Filemode]::Open,[System.IO.FileAccess]::Read)
  $res = [System.BitConverter]::ToString($md5CSP.ComputeHash($fptr))
  $fptr.Dispose()
  $res.Replace('-','')
}

### EOF