﻿#########################################################################################
<#
.Synopsis
    Loads L10Ned message table for given script

.DESCRIPTION
    This function finds and imports localized message table (*.psd1 file) that can (optionally) 
    be provided for each *.ps1 script file.

    If the table is found and succesfully imported, it gets integrated with a global table ($Global:g_msgTable)
    which consolidates all L10Ned resources for all the script files included in current PS session.
    The name of the newly added table is added to "LoadedResources" list which is used to avoid
    importing the same resources more than once.

.EXAMPLE
    Every *.ps1 file which outputs any text messages should have all the messages L10Ned 
    and stored in correspondent resource file (*.psd1). To load the messages, each *.ps1
    must have the following code added to (or near by) the top fo the file - 

    if (!(Get-Command -Name Load-L10NMessageTable -ea SilentlyContinue)) {
       . ".\agsLogTools.ps1"
    }
    Load-L10NMessageTable $MyInvocation.MyCommand.Name

.INPUTS
    Script file name 

.OUTPUTS
    None
      
.NOTES
    The resource file must contain a data section like this - 

ConvertFrom-StringData @'
ABCD_EXAMPLE_0 = This is a  message  with no parameters
ABCD_EXAMPLE_1 = This is a  message  with 1 parameter  : {0}
ABCD_EXAMPLE_2 = This is a  message  with 2 parameters : name={0}, value={1}
.  .  .  .   .  .
'@
    The message names must be globally unique. The uniqueness achieved by using
    a unique "prefix" (e.g. "ABCD") per each ps1 script file.
#>
function Load-L10NMessageTable($File)
{
  $msgTable = $null
  Import-LocalizedData -bindingVariable msgTable -FileName $File  -ErrorAction SilentlyContinue
  if (!$global:g_msgTable) {
    $global:g_msgTable = @{}
    $global:g_msgTable += @{LoadedResources = ""}
  }
  
  if (!$g_msgTable.LoadedResources.Contains($File)) { 
    if ($msgTable -and $msgTable.Count -gt 0) {
      $global:g_msgTable += $msgTable
      $g_msgTable.LoadedResources += $File + ";"
    }
  }
}

#########################################################################################
<#
.Synopsis
    Log file initialization and UN-initialization.

.DESCRIPTION
    To initialize    - pass in a fully qualified path/file name.
    To UN-initialize - use $null as input param
#>
function Log-Init([string]$LogFile) 
{
  if (!$LogFile) {
    $global:g_LogFile = $null
    log "logging to file disabled"
    return 
  }

  $logDir = Split-Path $LogFile
  if (!(Test-Path $logDir)) {
    New-Item -Path $logDir -Type directory -Force  | Out-Null
  }
    
  $global:g_LogFile = $LogFile
  log "logging to file enabled : {0}" -p $global:g_LogFile
}

#########################################################################################
<#
.Synopsis
    Returns path to the directory where log file is residing.

.DESCRIPTION
    If the log is not initialized yet (Log-Init), returns System drive root path 
#>
function Get-LogDir() 
{
  $dir = $null

  if ($global:g_LogFile) {
    $dir = Split-Path $global:g_LogFile
  } else {
    $dir = [System.Environment]::ExpandEnvironmentVariables("%SystemDrive%\")  
  }

  return $dir
}

#########################################################################################
<#
.Synopsis
    Sends text message to the host (console window) and saves a copy in the log file (optional)

.DESCRIPTION
    If the log is not initialized yet (Log-Init), returns System drive root path 
#>
function Log-Message 
{
  Param([Parameter(ValueFromPipeline=$true)][Alias("m")][string]$Msg="",
        [Alias("fc")][ConsoleColor]$ForegroundColor=([ConsoleColor]::White),
        [switch]$PassThru,
        [Alias("s")][switch]$Simple,
        [Alias("p")]$Params
        )

  Process 
  {
    ## TRICK: make sure the message is not a string code
    $Msg = Format-LocalizedString -c $Msg -p $Params
    $str = ""

    if ($Simple) {    # no timestamp, print the string as is 
      $str = $Msg
    } else {
      $str = [string]::Format("[{0}] {1}", (Get-Date -Format "MM-dd-yyyy HH:mm:ss.ffffff"), $Msg) 
    }

    if ($str -and $global:g_LogFile) {
      Add-Content -Path $global:g_LogFile -Value $str -Force -ErrorAction SilentlyContinue
    }

    Write-Host $str -ForegroundColor:$ForegroundColor
    if ($PassThru) { 
      return $Msg     ## NOTE: pass the message as-is
    } 
  }
}

#########################################################################################

<#
.Synopsis
    logs Verbose, Warning, Debug and Error automation records, 
    everything else is passed thru 

.DESCRIPTION

.EXAMPLE
    $res = function-name -Verbose  *>&1 | Log-Record
    $res = function-name -Verbose  *>&1 | logr
        - logs all the records to log file and send them to console window, output is 
          passed thru and assigned to $res

.EXAMPLE
    $res = function-name -Verbose  4>&1 | logr
        - logs only Verbose messages
#>
function Log-Record
{
#NO  [CmdletBinding()]
  Param ( [Parameter(ValueFromPipeline=$true)][Alias("obj")]$Object,[switch]$NoHost)
  Process 
  {
    if ($Object) 
    {
      $str = $null
      if ($Object -is [System.Management.Automation.VerboseRecord]) {
        $str = "VERBOSE: " + $Object.Message
        if (!$NoHost) {
          $VerbosePreference = "Continue"
          Write-Verbose $Object
        }
      } 
      elseif ($Object -is [System.Management.Automation.WarningRecord]) {
        $str = "WARNING: " + $Object.Message
        if (!$NoHost) {
          $WarningPreference =  "Continue"
          Write-Warning $Object
        }
      } 
      elseif ($Object -is [System.Management.Automation.DebugRecord]) {
        $str = "DEBUG: " + $Object.Message
        if (!$NoHost) {
          $DebugPreference = "Continue"
          Write-Debug $Object
        }
      } 
      elseif ($Object -is [System.Management.Automation.ErrorRecord]) {
        $str = "ERROR: " + ($Object | Out-String)
        if (!$NoHost) {
          $ErrorActionPreference = "Continue"
          Write-Error $Object
        }
      } 

      # save to file
      if ($str -and $global:g_LogFile) {
        Add-Content -Path $global:g_LogFile -Value $str -ErrorAction Continue -Force 
      }

      # return only if not processed 
      if ($str -eq $null) {
        $Object
      }
    }
  }
}

#################################################################################
##
function Log-NewLine([int]$NumLines=1)
{
  while ($NumLines -gt 0) {
    Log-Message "" -Simple
    $NumLines--
  }
}

#########################################################################################
##
function Log-PSBoundParams($Params=$PSBoundParameters,[string]$Caller)
{
  if (!$global:S_DEBUG) {
    return
  }

  $Caller = Get-ParentFunctionName

  $str = "$Caller  :  Bound parameters : `n" 
  if ($Params -and ($Params.Count -gt 0)) {
    $ht = @{}
    $Params.GetEnumerator() | %{if($_){$ht += @{"$($_.Key)"="$($_.Value)"}} }
    $str += $ht | ft -AutoSize | Out-String
  }
  
  Log-Message $str -Simple -fc DarkYellow
}

#########################################################################################
##
function Format-LocalizedString([Parameter(Position=1)][Alias("c")][string]$Code,[Alias("p")]$Params)   
{
  # If the 'code' is not found in the res file, use the code itself as the template
  $template = $Code
  if ($g_msgTable -and $g_msgTable.$Code) {
    $template = $g_msgTable.$Code
  }

  # TRICK! If there is no params, just return it as is
  $res = $template
  if ($Params) {
    $res = $template -f $Params
  } 
  return $res
}

#################################################################################
<#
.Synopsis
   generates a random string

.DESCRIPTION
   Generates a string of random digits, with Length limited by MaxLength (default value = 16).
   
   Calling this with -Stick makes it remember the last generated string, all subsequent calls 
   will produce exactly the same string.

   Call this function with -UnStick parameter to switch it back to generating new random strings
   for every new invocation.
   
.INPUTS

.OUTPUTS
    System.String

.NOTES
   Get-Random

.EXAMPLE
    PS C:\> Create-RandomString   
            - This call generates a random string "1170914801204543" of default length (16 chars).

.EXAMPLE
    PS C:\> Create-RandomString  -MaxLength 3 
        - this call generates a random string of length "3" (e.g. "586")

.EXAMPLE
    PS C:\> Create-RandomString -Stick 
        - generates a random string "3839113191109369" and makes it stuck, 
          any subsequent invokes will now return the same string 

          PS C:\> Create-RandomString   
          .  .  .  .
          PS C:\> Create-RandomString  -MaxLength 200 
                - all  return the same string "3839113191109369" 
                  NOTE: Since new string is not generated, parameter -MaxLength is ignored
.EXAMPLE
    PS C:\> Create-RandomString -Unstick  -MaxLength 5 
        - turns the "sticky" mode OFF and returns a new random string of length 5

.EXAMPLE
    PS C:\> Create-RandomString -Unstick  -MaxLength 5 -Stick
        - turns the "sticky" mode OFF, generates a new random string of length 5 and 
          then turns the sticky mode back ON. 
#>
function Create-RandomString([int]$MaxLength=16,[switch]$Stick,[switch]$UnStick) 
{
  if ($UnStick) {
    $global:g_agsStickyRandomString = $null
  }

  # if the string was once made "sticky" - keep returning the same string (until called with -Unstick)
  if ($global:g_agsStickyRandomString) {
    return $global:g_agsStickyRandomString
  }

  if ($MaxLength -lt 1) {
    throw (fls "Invalid value MaxLength input parameter : {0}" -p $MaxLength)
  }

  $str = ""   # random part
  do {
    $str += (Get-Random).toString()
  } while ($str.Length -lt $MaxLength) 

  $result = $str.Substring(0,$MaxLength)

  if ($Stick) {
    $global:g_agsStickyRandomString = $result
  }
  
  return  $result
}

#################################################################################
##               Aliases
#################################################################################
##
Set-Alias log  Log-Message
Set-Alias logr Log-Record
Set-Alias lnl  Log-NewLine
Set-Alias fls  Format-LocalizedString
Set-Alias rnds Create-RandomString

### EOF
